// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// ClassName		:	Application
// Version			:	0.10
// Type				:	Swing application
// ThreadSafe		:	Yep
// Description		:
//						Short description.
// URL				:	<http://www.nokrew.org/skeletons>
// ----------------------------------------------------------------------------
// Authors			:	Sbastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	15-Sep-2001	
// Last mod.		:	24-Oct-2001	
// History			:	
//						24-Sep-2001 Moved inside the app package, better doc
//						and structure.(sp)
//						15-Sep-2001 First implementation.(sp)
//
// Bugs				:
//						-
//
// To do				:
//						-

package net.sourceforge.spedit.app;

import java.io.*;
import java.util.*;
import java.awt.Component;
import java.awt.event.*;
import java.lang.reflect.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.View;
import javax.swing.text.CompositeView;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.python.util.*;
import org.python.core.PyObject;
import net.sourceforge.spedit.ui.XmFormLayout;
import net.sourceforge.spedit.core.*;
import net.sourceforge.spedit.ui.*;

//-----------------------------------------------------------------------------
//
// Application 
//
//-----------------------------------------------------------------------------

/**
  * The <code>Application</code> class is at the heart of the editor. It stores
  * references to openened documents and provide event hooking mechanisms.
*/
public class Application
{
	public static final PythonInterpreter python = new PythonInterpreter();
	protected DocumentFrame currentDocumentFrame;
	protected Vector documentFrames;


	//-------------------------------------------------------------------------
	//
	// Constructors
	//
	//-------------------------------------------------------------------------

	public Application()
	{
		documentFrames = new Vector();
		currentDocumentFrame = new DocumentFrame();
		documentFrames.add(currentDocumentFrame);
	}

	public void init()
	{
		currentDocumentFrame.init();
	}

	//-------------------------------------------------------------------------
	//
	// Document methods
	//
	//-------------------------------------------------------------------------


	public void setCurrentDocument(XMLDocument doc)
	{
		if ( currentDocumentFrame!=null )
		{
			currentDocumentFrame.setDocument(doc);
		}
	}

	//-------------------------------------------------------------------------
	//
	// Document frame
	//
	//-------------------------------------------------------------------------

	/**
	  * The document frame represent a document window on the screen. It
	  * basically holds a navigation tree view, menu bar and of course document
	  * editing panel.
	*/
	public class DocumentFrame extends JFrame implements MouseListener
	{
		// PROTECTED___________________________________________________________
		protected JEditorPane    editorPane;
		protected XMLEditorKit   editorKit;
		protected XMLDocument    document;
		protected JScrollPane    scrollPane;
		protected JSplitPane     splitPane;
		protected NavigationTree navigator;
		protected XMLViewFactory factory;
		//OldWrapper is the previously selected wrapper, see mouseClicked.
		protected WrapperView    oldWrapper=null;

		private String[] constraints = {
			"document.top=form.top",
			"document.bottom=form.bottom-20",
			"document.right=form.right",
			"document.left=form.left",
			"status.top=document.bottom",
			"status.bottom=form.bottom",
			"status.right=form.right",
			"status.left=form.left"
		};

		//----------------------------------------------------------------------
		//
		// Constructor
		//
		//----------------------------------------------------------------------

		public DocumentFrame()
		{
			super("Document");
			editorPane = new JEditorPane();
			navigator = new NavigationTree(this);
			//editorKit = new XMLEditorKit("../Support/Configuration.xml");
			editorKit = new XMLEditorKit("../Support/SP-Configuration.xml");
			editorPane.setEditorKit(editorKit);
			factory = (XMLViewFactory)editorKit.getViewFactory();
			editorPane.addMouseListener(this);
			init();
		}

		public void init()
		{			
			//We load the custom file
			File ff = new File("../Support/SP-Document.xml");
			//File ff = new File("../Support/Document.xml");
			document = XMLDocument.loadFromFile(ff, factory);
			document.filter(factory);
			setDocument(document);

			//We set layout for container
			getContentPane().setLayout(new XmFormLayout(constraints));
		
			python.execfile("../Support/Application.py");
			python.set("frame", this);
			python.exec("listener = MenuListener(frame)");
			python.eval("frame.setJMenuBar(buildMenus(menus, listener))");
			python.exec("status = StatusBar(frame)");

			//We generate the document content
			scrollPane = new JScrollPane(editorPane);
			JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
			new JScrollPane(navigator), scrollPane);
			splitPane.setName("document");
			getContentPane().add("document", splitPane);
			python.eval("frame.getContentPane().add('status', status)");
			
			//We open the window
			setBounds(10,10,500,400);
			splitPane.setDividerLocation(0.40);
			setVisible(true);
		}

		//----------------------------------------------------------------------
		//
		// Event handling
		//
		//----------------------------------------------------------------------

		// DOCUMENT____________________________________________________________

		// MOUSE_______________________________________________________________
		
		public void mouseClicked(MouseEvent evt)
		{
			XMLElementWrapper e = (XMLElementWrapper)getDocument().getDefaultRootElement();
			WrapperView wrap = (WrapperView)factory.getElementView(e);
			wrap = wrap.getViewAt((float)evt.getX(), (float)evt.getY());			
			if ( oldWrapper!=null )
			{	oldWrapper.setFocused(false); }
			wrap.setFocused(true);
			oldWrapper = wrap;
			wrap.propagateMouseClicked(evt);
			
			if (evt.isControlDown() || SwingUtilities.isRightMouseButton(evt))
			{
				int dot = editorPane.getCaret().getDot();
				javax.swing.text.Element elt = getDocument().getElementAt(dot);
				//We make sure to have a parent element. All nodes that are not
				//XMLElementWrappers have a parent that is an XMLElementWrapper.
				if ( !(elt instanceof XMLElementWrapper) )
				{	elt = elt.getParentElement(); }
				XMLElementWrapper wrp = (XMLElementWrapper)elt;
				JPopupMenu menu = getElementMenu(wrp,dot);
				if (menu!=null)
				{
					//TODO: Should adjust the x and y position according to the
					//screen bounds.
					menu.show(editorPane, evt.getX(), evt.getY());
				}
			}
		}

		public void mouseEntered(MouseEvent evt)
		{
		}

		public void mouseExited(MouseEvent evt)
		{
		}

		public void mousePressed(MouseEvent evt)
		{
		}

		public void mouseReleased(MouseEvent evt)
		{
		}

		//----------------------------------------------------------------------
		//
		// Document-related methods
		//
		//----------------------------------------------------------------------


		public void setDocument(XMLDocument doc)
		{
			editorPane.setDocument(doc);
			navigator.setDocument(doc);
		}

		public XMLDocument getDocument()
		{
			return (XMLDocument)editorPane.getDocument();
		}

		public JEditorPane getEditorPane()
		{
			return editorPane;
		}

		public void saveDocument(File f)
		{
			try{ document.saveToFile(f); }
			catch (Exception e) {e.printStackTrace();}
		}

		/**
		  * Creates a new <code>JPopupMenu</code> from the given element, which
		  * can only be an <code>XMLElementWrapper</code>. The different
		  * menu items are constructed using the information embedded inside
		  * this element (an <code>XMLInformation</code> instance.)
		  * @param element The element from which the menu will be generated
		  * @param dot The position in the element at which the menu will consider
		  * operations such as insertion and removal.
		  * @return The generated menu.
		*/
		public JPopupMenu getElementMenu(XMLElementWrapper element, int dot)
		{
			//TODO: Check the *dot* value to be in the element range
			JPopupMenu menu = new JPopupMenu("Element menu");

			//We get the information of the current element, or the parent
			XMLInformation info  = element.getInformation();
			if ( info!=null )
			{
				JMenuItem item;
				Enumeration e = info.getChildren();
				menu.add(new ActionElementRemove(element));
				menu.add(new JSeparator());
				//We add insertable children in the menu
				while (  e.hasMoreElements() )
				{
					//(item = new JMenuItem("Insert "+e.nextElement());
					//item.setActionCommand("add");
					XMLElement nelt = new XMLElement(e.nextElement().toString());
					menu.add(new ActionElementInsert(element,nelt, dot));
				}

				return menu;
			}
			else
			{
				return null;
			}
		}

		//----------------------------------------------------------------------
		//
		// Remove an element
		//
		//----------------------------------------------------------------------

		protected class ActionElementRemove extends AbstractAction
		{
			protected XMLElementWrapper wrapper;

			public ActionElementRemove(XMLElementWrapper wrp)
			{
				super("Remove current element...");
				wrapper = wrp;
			}

			public void actionPerformed(ActionEvent evt)
			{
				XMLElementWrapper wrp = (XMLElementWrapper)wrapper.getParentElement();
				CompositeView view =
				(CompositeView)((WrapperView)factory.getElementView(wrp)).getNestedView();
				wrp.remove(wrapper.getStartOffset(), wrapper.getEndOffset());
				//We do this to force a loadChildren() method call in the CompositeView.
				view.replace(0,view.getViewCount(),new View[0]);
				view.setParent(view.getParent());
				view.getContainer().repaint();
			}
		}

		//----------------------------------------------------------------------
		//
		// Insert an element
		//
		//----------------------------------------------------------------------

		protected class ActionElementInsert extends AbstractAction
		{
			protected XMLElementWrapper wrapper;
			protected XMLElement newElement;
			protected int pos;

			//TODO: Should take a Position object instead of a number
			public ActionElementInsert(XMLElementWrapper wrp,
			XMLElement elt, int pos)
			{
				super("Insert "+elt.getName());
				this.wrapper = wrp;
				this.newElement = elt;
				this.pos = pos;
			}

			public void actionPerformed(ActionEvent evt)
			{
				XMLElementWrapper wrp = (XMLElementWrapper)wrapper.getParentElement();
				//We get the parent view
				CompositeView view =
				(CompositeView)((WrapperView)factory.getElementView(wrp)).getNestedView();
				//We do this to enable a forceed reload of the composite view children
				view.replace(0,view.getViewCount(),new View[0]);
				//FIXME: Insertion should be made through the DOM...
				wrapper.insert(newElement,pos,javax.swing.text.Position.Bias.Forward);
				//This makes the actual forced reaload after element insertion.
				view.setParent(view.getParent());
				view.getContainer().repaint();
			}
		}

	}

	//-------------------------------------------------------------------------
	//
	// Element menu event handling
	//
	//-------------------------------------------------------------------------


	public static void main(String[] args)
	{
		System.out.println("SPEdit - Prototype");
		Application app = new Application();
	}
}

// EOF-Linux/ISO-8859-1-------------------------------@RisingSun//Java//1.0//EN
