// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// ClassName		:	XMLAttributes
// Version			:	0.76
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	Subclass of JDOM Element, plus a Wrapper to make the
//						JDOM element visible as a Swing Element.
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	17-Sep-2001	
// Last mod.		:	17-Sep-2001	
// History			:
//						17-Sep-2001 Moved from XMLElement.(SP)
//
// Notes			:
//
// Bugs				:
//					  -
//
// To do			:
//					  -Make it threadsafe without synchronizing, like in every
//					  method that iterates. Make getStartOffset and
//					  getEndOffset synchronized.(SP)
//					  -Constructors suck.(SP)
//					  -Implement parent attribute resolution in XMLAttributes 
//					   (SP).

package net.sourceforge.spedit.core;

import java.util.*;
import javax.swing.text.*;
import javax.swing.tree.TreeNode;
import org.jdom.CDATA;

//---------------------------------------------------------------------
//
// XMLAttributes
//
//---------------------------------------------------------------------

/**
  * The <code>XMLAttributes</code> class has the same role as the
  * <code>XMLElementWrapper</code>, it wraps the attributes of a JDOM
  * element and present the methods defined in the <code>AttributeSet</code>
  * interface of the Swing text package.<br/>
  * This class is needed by the <code>XMLElementWrapper</code> class.
  * @version 0.72
  * @author Sebastien Pierre
*/
public class XMLAttributes implements AttributeSet
{

	protected List attributes;

	public XMLAttributes(List attributes)
	{
		setAttributes(attributes);
	}

	public synchronized void setAttributes(List attributes)
	{
		this.attributes = attributes;
	}

	/**
	  * Returns true if this set contains this attribute with an equal
	  * value. 
	*/
	public boolean containsAttribute(Object name, Object value)
	{
		ListIterator it=attributes.listIterator();
		org.jdom.Attribute attrib;

		while ( it.hasNext() )
		{
			attrib = (org.jdom.Attribute)it.next();
			if (attrib.getName().equals(name)
			&& attrib.getValue().equals(value))
			{	return true;	}
		}

		return false;
	}

	/**
	  * Returns true if this set contains all the attributes with equal
	  * values. This should be used instead of equals() which will not
	  * work with the JDom implementation of the attributes.
	*/
	public boolean containsAttributes(AttributeSet attrs)
	{
		ListIterator it=attributes.listIterator();
		org.jdom.Attribute attrib;

		while ( it.hasNext() )
		{
			attrib = (org.jdom.Attribute)it.next();
			if (!attrs.containsAttribute(attrib.getName(),attrib.getValue()))
			{	return false;	}
		}

		return true;
	}
	/**
	  * Returns an attribute set that is guaranteed not to change over
	  * time. 
	*/
	public AttributeSet copyAttributes()
	{
		return (AttributeSet)this.clone();
	}

	/**
	  * Fetches the value of the given attribute. If the value is not found
	  * locally, the search is continued upward through the resolving
	  * parent (if one exists) until the value is either found or there are
	  * no more parents. If the value is not found, null is returned. 
	*/
	public Object getAttribute(Object name)
	{
		ListIterator it=attributes.listIterator();
		org.jdom.Attribute attrib;

		while ( it.hasNext() )
		{
			attrib = (org.jdom.Attribute)it.next();
			if (attrib.getName().equals(name))
			{	return attrib.getValue(); }
		}

		return null;
	}

	/**
	  * Returns the number of attributes contained in this set.
	*/
	public int getAttributeCount()
	{ return attributes.size(); }

	/**
	  * Returns an enumeration over the names of the attributes in the set.
	*/
	public Enumeration getAttributeNames()
	{ return new XMLAttributesEnumeration(attributes); }

	/**
	  * Gets the resolving parent.  AttributeSet can be used hierarchically
	  * and if the current AttributeSet doesn't find the required element,
	  * it will look inside its parent for the specified attribute until
	  * the root parent is
	  * reached.
	*/
	//TODO: Implement the resolving parent
	public AttributeSet getResolveParent()
	{	return null; }

	/**
	  * Create a deep copy of the current attribute set.  This method is
	  * used by the copyAttributes method, which guaratees that the new
	  * AttributeSet will not change if you change the original values.
	*/
	public Object clone()
	{
		XMLAttributes attr = new XMLAttributes(new LinkedList());

		org.jdom.Attribute attrib;
		ListIterator it=attributes.listIterator();

		while ( it.hasNext() )
		{
			attrib = (org.jdom.Attribute)it.next();
			attr.attributes.add(new
			org.jdom.Attribute((String)attrib.getName(),
			  (String)attrib.getValue()));
		} 

		return attr;
	}
	
	/**
	  *  Checks whether the named attribute has a value specified in the
	  *  set without resolving through another attribute set. 
	*/
	public boolean isDefined(Object name)
	{
		ListIterator it=attributes.listIterator();
		org.jdom.Attribute attrib;

		while ( it.hasNext() )
		{
			attrib = (org.jdom.Attribute)it.next();
			if (attrib.getName().equals(name))
			{	return true;	}
		} 

		return false;
	}

	/**
	  * Determines if the two attribute sets are equivalent.
	*/
	public boolean isEqual(AttributeSet attrs)
	{
		ListIterator it=attributes.listIterator();
		org.jdom.Attribute attrib;

		while ( it.hasNext() )
		{
			attrib = (org.jdom.Attribute)it.next();
			if (!attrs.containsAttribute(attrib.getName(),
			attrib.getValue()))
			{	return false;	}
		}

		return true;
	}

	public class XMLAttributesEnumeration implements Enumeration
	{
		ListIterator it;

		public XMLAttributesEnumeration(List attr)
		{
			it = attr.listIterator();
		}

		public boolean hasMoreElements()
		{	return it.hasNext();	}

		public Object nextElement()
		{
			return ((org.jdom.Attribute)it.next()).getValue();
		}
	}

	public String toString()
	{
		String result="XMLAttributes ["+hashCode()+"]\n";
		ListIterator it=attributes.listIterator();

		if (!it.hasNext())
		{	result+="  is empty."; }

		while (it.hasNext())
		{
			org.jdom.Attribute att = (org.jdom.Attribute)it.next();
			result+="  "+att.getName()+"\t\t= "+att.getValue();
		}
		return result;
	}
}
// EOF-Unix/ASCII--------------------------------------@RisingSun//Java/1.0//EN
