// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// ClassName		:	XMLElementWrapper
// Version			:	0.93c
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	Subclass of JDOM Element, plus a Wrapper to make the
//						JDOM element visible as a Swing Element.
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sbastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	17-Sep-2001	
// Last mod.		:	01-Nov-2001	
// History			:
//						01-Nov-2001 Fixed nasty bug in the insert method.(sp)
//						26-Oct-2001 Added some Javadoc. (sp)
//						17-Oct-2001 Code cleanup, optimisation of offset
//						access, optimisation of event handling. Started to make
//						everythin threadsafe.(SP)
//						03-Oct-2001 Fixed bug in event positionning.(SP)
//						24-Sep-2001 Improved event firing, added some comments
//						in the code to make it clearer.(SP)
//						20-Sep-2001 Cosmetic updates. Added return types for
//						event handling.(SP)
//						19-Sep-2001 Made String and CDATA appear as
//						XMLTextWrapper nodes in the Swing document model.(SP)
//						17-Sep-2001 Moved from XMLElement.(SP)
//
// Notes			:
//
// Bugs				:
//					  -
//
// To do			:
//					  - Insert(string) should take care of the Position.Bias
//					  - There may be a threadsafe problem in child loops, as
//					    the condition is a variable and note the
//					    parent.getContent().size()... should test the
//					    difference between variable and method calls.

package net.sourceforge.spedit.core;

import java.util.*;
import javax.swing.text.*;
import javax.swing.tree.TreeNode;
import org.jdom.CDATA;


//---------------------------------------------------------------------------
//
// XMLElementWrapper
//
//----------------------------------------------------------------------------

/**
  * The <code>XMLElementWrapper</code> allows a JDOM <code>Element</code>
  * to appear a Swing <code>Element</code>. This wrapper does not work for
  * JDOM <code>Element</code> but works fine with the
  * <code>XMLElement</code> subclass which has been designed for this
  * purpose.
  * <p>It is important to note that the <code>XMLElementWrapper</code>
  * automatically notifies the document of a given element when it changes its
  * content.</p>
  * @version 0.93
  * @author Sebastien Pierre
*/
public class XMLElementWrapper implements Element, TreeNode, XMLNode
{

	// PUBLIC__________________________________________________________

	/**
	  * Means that the searched element index is located before the current
	  * element.
	*/
	public static final short ELEMENT_BEFORE=222;

	/**
	  * Means that the searched element index is located after the current
	  * element.
	*/
	public static final short ELEMENT_AFTER=111;

	/**
	  * Means that the searched element index does correspond to the text
	  * directly contained in the wrapped <code>XMLElement</code>, and not in a
	  * child <code>XMLElement</code>.
	*/
	public static final short ELEMENT_THIS=111;
	/**

	  * Indicates that the given block should be cut left.
	*/
	public static final short CUT_LEFT = 1;

	/**
	  * Indicates that the given block should be cut right.
	*/
	public static final short CUT_RIGHT = 2;

	/**
	  * Indicates that the given block should be cut inside.
	*/
	public static final short CUT_INSIDE = 3;

	/**
	  * Indicates that the given block should be deleted.
	*/
	public static final short DELETE = 4;

	/**
	  * Indicates that the given block should be left as is.
	*/
	public static final short LEAVE = 5;

	// PROTECTED______________________________________________________________
	protected XMLElement     element;
	protected XMLInformation information;
	protected int            contentLength;
	protected Position       startPosition = null;
	protected Position       endPosition = null;

	// CONSTRUCTORS___________________________________________________________

	/**
	  * Constructs a new <code>XMLElemetWrapper</code> fron the given
	  * <code>XMLElement</code>, which is a JDOM <code>Element</code> sublass.
	*/
	public XMLElementWrapper(XMLElement element)
	{
		this.element = element;
		information = null;
	}

	//------------------------------------------------------------------------
	//
	// XMLInformation
	//
	//------------------------------------------------------------------------

	/**
	  * Returns the information attached to this element. Information includes
	  * which views are recommended for this element, but also information
	  * about the possible children of this elment.
	  * @return The associated <code>XMLInformation</code> instance.
	*/
	public XMLInformation getInformation()
	{
		return information;
	}

	/**
	  * <p>Sets the information attached to this particular element. In general
	  * elements of the same namespace and name share the same information
	  * reference. Using this method will set a specific XML information to
	  * this element.</p>
	  * <p>You generally should'nt use this method without good reasons
	  * <code>;)</code>.</p>
	  * @param info The new information that will be attached to this element.
	*/
	public void setInformation(XMLInformation info)
	{
		information = info;
	}

	/**
	  * Tells wether this elmement has available information.
	  * @return True if has, false if it has no available information.
	*/
	public boolean hasInformation()
	{
		if ( information!=null )
		{	return true;	}
		else
		{	return false;	}
	}

	//------------------------------------------------------------------------
	//
	// Tree node compliance
	//
	//------------------------------------------------------------------------

	public synchronized Enumeration children() 
	{
		Vector result = new Vector();
		ListIterator it = element.getContent().listIterator();
		while (it.hasNext())
		{
			Object o=it.next();
			if ( o instanceof XMLElement )
			{	result.add(((XMLElement)o).getWrapper());	}
			else if ( o instanceof XMLText )
			{	result.add(((XMLElement)o).getWrapper());	}
		}
		return result.elements();
	}

	public boolean getAllowsChildren() 
	{
		return true;
	}

	public synchronized TreeNode getChildAt(int childIndex) 
	{
		return (TreeNode)getElement(childIndex);
	}

	public int getChildCount() 
	{
		return getElementCount();
	}

	public int getIndex(TreeNode node) 
	{
		ListIterator it = element.getContent().listIterator();
		for (int i=0 ; it.hasNext() ; i++)
		{
			Object o=it.next();
			if ( o instanceof XMLElement && ((XMLElement)o).getWrapper().equals(node) )
			{	return i;	}
			else if ( o instanceof XMLText && ((XMLText)o).getWrapper().equals(node) )
			{	return i;	}
		}

		return -1;
	}

	public TreeNode getParent() 
	{
		if ( element.getParent()!=null )
		{	return ((XMLElement)element.getParent()).getWrapper();	}
		else
		{	return null;	}
	}

	//------------------------------------------------------------------------
	//
	// Element accession
	//
	//------------------------------------------------------------------------

	/**
	  * Gets the number of child elements contained by this element.
	*/
	public int getElementCount() 
	{
		return element.getContent().size();
	}

	/**
	  * <p>Returns the <code>Element</code> that is the nth of this
	  * <code>Element</code>.</p>
	  * <p><em>Note:</em> The element indexes are different from the wrapped
	  * element indexes, as the wrapper only counts elements that are
	  * XMLElements.</p>
	  * @param <code>index</code> The element's index in the wrapper child
	  * list.
	  * @return The <code>Element</code> which is the nth child of the
	  * wrapper, can be <code>null</code> if not found.
	*/
	public synchronized Element getElement(int index) 
	{ 
		int counter = 0;
		Object object = element.getContent().get(index);

		if (object instanceof XMLElement)
		{
			return ((XMLElement)object).getWrapper();
		}
		else if (object instanceof XMLText)
		{
			return ((XMLText)object).getWrapper();
		}
		else
		{
			System.err.println("Unrecognized child type: "+object.getClass());
			return null;
		}
	}

	/**
	  * Searches any descendant element that contains the given
	  * <code>offset</code>. This method is an extension of the
	  * <code>getElement</code> method, which only returns the nth element of
	  * the <code>XMLElement</code> content.
	  * @param <code>offset</code> is the offset of the element in the text. If
	  * this offset is lower than <code>getStartOffset()</code> or higher or
	  * equal to <code>getEndOffset()</code> then <b>null</b> is returned.
	  * @return The <code>Element</code> or <b>null</b> is <code>offset</code>
	  * is out of the XMLElement bounds.
	*/
	public Element getElementAt(int offset)
	{
		if ( offset<getStartOffset() || offset>=getEndOffset() )
		{
			System.err.println("XMLElementWrapper: Offset out of bounds "
			+offset+" for element "+this);
			return null;
		}
		else
		{
			int elem_index = getElementIndex(offset);
			Object elem = getElement(elem_index);

			if (elem instanceof XMLElementWrapper)
			{
				return ((XMLElementWrapper)elem).getElementAt(offset);
			}
			else if (elem instanceof XMLTextWrapper)
			{
				return (XMLTextWrapper)elem;
			}
			else
			{
				System.err.println("Unknown child type for XMLElementWrapper: "+elem.getClass());
				return null;
			}
		}
	}

	/**
	  * Gets the child element index closest to the given offset. The offset
	  * is specified relative to the beginning of the document. Returns -1 if
	  * the Element is a leaf, otherwise returns the index of the Element that
	  * best represents the given location. Returns 0 if the location is less
	  * than the start offset. Returns getElementCount() - 1 if the location is
	  * greater than or equal to the end offset. 
	  * @param <code>offset<code> - the specified offset >= 0
	  * @return Returns: the element >= 0
	*/
	public int getElementIndex(int offset) 
	{
		int		coffset=getStartOffset();
		//This is to keep track of the element index. Also starts at -1
		//because we increment whenever we find an XMLElement.
		Object	current=null;

		//We check the bounds of the offset
		if ( offset<getStartOffset() )
		{	offset=getStartOffset();	}
		else if ( offset>=getEndOffset() )
		{	offset=getEndOffset();	}

		int i=0;
		//Then we look for the searched XMLElement
		for (i=0 ; i<element.getContent().size() ; i++)
		{
			current = element.getContent().get(i);

			//We can only have XMLElement, CDATA or Strings...
			if (current instanceof XMLElement)
			{
				coffset+=((XMLElement)current).getWrapper().getLength();
			}
			else if (current instanceof XMLText)
			{
				coffset+=((XMLText)current).getValue().length();
			}
			//This is just to make sure...
			else
			{
				System.err.println("Unknown content "+current);
			}

			//If we have reached the offset we wanted, then we return the
			//XMLElement index int the Wrapper.
			if ( offset<coffset )
			{
				return i;
			}
		}

		//We do so because i==getContent().size()
		return i-1;
	}

	/**
	  * Gets the index of the given object in the child list of this element.
	  * @return -1 if the element is not a child, otherwise its index.
	*/
	public int getElementIndex(Object object) 
	{
		//This is to keep track of the element index. Also starts at -1
		//because we increment whenever we find an XMLElement.
		Object	current=null;

		//We check the bounds of the offset
		//Then we look for the searched XMLElement
		for (int i=0 ; i<element.getContent().size() ; i++)
		{
			current = element.getContent().get(i);

			//We can only have XMLElement, CDATA or Strings...
			if (current.equals(object))
			{ return i; }
		}
		return -1;
	}

	//------------------------------------------------------------------------
	//
	// Offsets
	//
	//------------------------------------------------------------------------

	/**
	  * Fetches the offset from the beginning of the document that this
	  * element begins at. When the node has no parent or no document, then the
	  * returned value is <code>0</code>
	  * @return The starting offset in the containing document, or
	  * <code>0</code> in case the element is not attached to a document.
	*/
	public int getStartOffset() 
	{
		if ( element.getParent()==null )
		{
			return 0;
		}
		else
		{
			//Is the cached startOffset outdated ?
			//if ( getDocument()==null ||((XMLDocument)element.getDocument()).timeStamp>element.timeStamp )
			{
				element.startOffset =
				((XMLElement)element.getParent()).getWrapper().getStartOffset();
				ListIterator it = ((XMLElement)element.getParent()).getContent().listIterator();
				while (it.hasNext())
				{
					Object o=it.next();
					if ( o instanceof XMLElement )
					{
						XMLElement elt = (XMLElement)o;
						if ( elt==element )
						{	return element.startOffset; }
						else
						{	element.startOffset+=elt.getWrapper().getLength(); } 
					}
					else if ( o instanceof XMLText )
					{ element.startOffset+=((XMLText)o).getValue().length(); }
				}
			}
			return element.startOffset;
		}

		/*
		if ( element.getParent()==null )
		{
			return 0;
		}
		else if ( getDocument()==null )
		{
			return 0;
		}
		else if ( startPosition!=null )
		{
			return startPosition.getOffset();
		}
		else
		{
			XMLDocument doc   = (XMLDocument)getDocument();
			XMLElement parent = (XMLElement)element.getParent();
			int startOffset   = parent.getWrapper().getStartOffset();

			//We iterate through the parent children
			for ( int i=0 ; i<parent.getContent().size() ; i++ )
			{
				Object o = parent.getContent().get(i);
				//If child is an XMLElement
				if ( o instanceof XMLElement )
				{
					XMLElement elt = (XMLElement)o;
					if ( elt==element )
					{	break;	}
					else
					{	startOffset+=elt.getWrapper().getLength(); } 
				}
				//If child is an XMLElement
				else if ( o instanceof XMLText )
				{
					startOffset+=((XMLText)o).getValue().length();
				}
				//Otherwise there is an unsupported child type, so we warn the
				//user.
				else
				{
					System.err.println("Warning: XMLElementWrapper.getStartOffset");
					System.err.println("         Element child not supported:"+
					o.toString());
				}
			}
			startPosition = doc.createBackwardPosition(startOffset);
			return startOffset;
		}
		*/
	}

	/**
	  * Fetches the offset from the beginning of the document that this
	  * element ends at.
	  * @return The starting offset in the containing document, or
	  * <code>0</code> in case the element is not attached to a document.
	*/
	public int getEndOffset() 
	{
		return getStartOffset()+getLength();
		/*
		//If the end position has already been created, we use it
		if ( endPosition!=null )
		{
			return endPosition.getOffset();
		}
		else if ( getDocument()==null )
		{
			return 0;
		}
		//Otherwise we create it
		else
		{
			XMLDocument doc   = (XMLDocument)getDocument();
			int contentLength=0;

			//We iterate through the element children to calculate the total
			//length of the children.
			for ( int i=0 ; i<element.getContent().size() ; i++ )
			{
				Object elt = element.getContent().get(i);
				System.out.println("Content element : "+elt);
				//The child is an XMLElement
				if ( elt instanceof XMLElement )
				{
					System.out.println(" Found child : "+elt);
					int old = contentLength;
					contentLength+=((XMLElement)elt).getWrapper().getLength();
					old = contentLength-old;
					System.out.println(" of length : "+old);
				}
				//The child is an XMLText
				else if ( elt instanceof XMLText)
				{
					System.out.println(" Found text : "+elt);
					int old = contentLength;
					contentLength+=((XMLText)elt).getValue().length();
					old = contentLength-old;
					System.out.println(" of length : "+old);
				}
				//The child is of an unknown type.
				else
				{
					System.err.println("Warning: XMLElementWrapper.getEndOffset");
					System.err.println("         Element child not supported:"+
					elt.toString());
				}
			}
			endPosition = doc.createForwardPosition(getStartOffset());
			return endPosition.getOffset();
		}
		*/
	}

	/**
	  * Returns the length of this element, int terms of text.
	  * This is simply the difference between <code>getEndOffset()</code>
	  * and <code>getStartOffset()</code>.
	*/
	public int getLength()
	{
		//We check if the cached attributes values are
		//still valid
		//if ( element.getDocument()==null || ((XMLDocument)element.getDocument()).timeStamp>element.timeStamp )
		{
			element.contentLength=0;
			ListIterator iterator = element.getContent().listIterator();
			while( iterator.hasNext() )
			{
				Object elt = iterator.next();
				if ( elt instanceof XMLElement )
				{
					element.contentLength+=((XMLElement)elt).getWrapper().getLength();
				}
				else if ( elt instanceof XMLText)
				{
					element.contentLength+=((XMLText)elt).getValue().length();
				}
				else
				{
					System.err.println("Unknown content: "+elt);
				}
			}
		}
		return element.contentLength;

		//return getEndOffset()-getStartOffset();
	} 

	/**
	  * Returns the start offset of the given object. This is used for instance
	  * by the <code>XMLText</code> to get its start offset. If the object was
	  * not a child of the current element, then <code>-1</code> is returned.
	  * @param object The object from which we want the start offset.
	  * @return The actual offset or <code>-1</code> if the object is not a
	  * child of the current element.
	*/
	public int getStartOffsetOf(Object object) 
	{
		int startOffset = getStartOffset();
		ListIterator it = ((XMLElement)element).getContent().listIterator();

		while (it.hasNext())
		{
			Object o=it.next();
			int increment=0;
			if ( o instanceof XMLElement )
			{
				XMLElement elt = (XMLElement)o;
				increment =elt.getWrapper().getLength();
			}
			else if ( o instanceof XMLText )
			{ increment=((XMLText)o).getValue().length(); }

			if ( object.equals(o) )
			{	return startOffset;	}

			startOffset+=increment;
		}
		return -1;

		/*
		int startOffset = getStartOffset();
		List content    = ((XMLElement)element).getContent();

		//We iterate through the children
		for (int i=0 ; i<content.size() ; i++ )
		{
			int increment=0;
			Object o=content.get(i);
			
			//Here we test recognized elements, but we do not warn of
			//unrecognized ones, because they should already be signaled by
			//start and end offset accesses.
			if ( o instanceof XMLElement )
			{
				XMLElement elt = (XMLElement)o;
				increment =elt.getWrapper().getLength();
			}
			else if ( o instanceof XMLText )
			{	increment=((XMLText)o).getValue().length(); }

			//Here we break if we have found the element
			if ( object.equals(o) )
			{	return startOffset;	}
			//Otherwise we increment thes start offset
			else
			{	startOffset+=increment; }
		}
		//If nothing was found we return -1
		return -1;
		*/
	}

	/**
	  * Tells whether the given range is contained withing the element.
	*/
	public boolean contains(int start, int end)
	{
		if (start<getEndOffset() && end>=getStartOffset())
		{	return true;	}
		else
		{	return false;	}
	}

	//------------------------------------------------------------------------
	//
	// Text access
	//
	//------------------------------------------------------------------------


	/**
	  * Inserts the given string after or before the given offset. In case
	  * the offset is out of the bounds of the current element, nothing
	  * will be done.
	  * @param <code>s</code> is the text that will be inserted.
	  * @param <code>offset</code> is the offset at which the string will
	  * be inserted. Attention, the offset begins at the 
	  * <code>getStartOffset()</code> value.
	  * @param <code>bias</code> tells wether the text has to be prepended,
	  * which is <code>Position.Bias.backward</code>, or appended, which is
	  * <code>Position.Bias.forward.</br>For example, if you
	  * <code>insert("X",2,Position.Bias.forward)<code> into
	  * <code>"ABCDE"</code>, you'll get <code>ABCXDE</code>, you would
	  * have get other <code>ABXCDE</code>
	*/
	//TODO: The event implementation is quite dirty, maybe some revamp would be
	//cool.
	public void insert( String s, int offset, Position.Bias bias )
	{
		if ( !contains(offset,offset+1) )
		{	return;	}

		//Variables
		int cur_offset=getStartOffset();
		XMLDocument.DocumentEvent event;
		XMLDocument doc = (XMLDocument)element.getDocument();
		ArrayList new_content = new ArrayList(element.getContent().size()-1);
		Vector    to_be_detached = new Vector();

		//Looking for the insertion point
		for ( int i=0 ; i<element.getContent().size() ; i++ )
		{
			int increment = 0;

			Object obj = element.getContent().get(i);
			if ( obj instanceof XMLElement )
			{
				XMLElementWrapper wrp = ((XMLElement)obj).getWrapper();
				increment = wrp.getLength();
				//The element contains the offset so we recurse
				if (cur_offset<=offset && offset<cur_offset+increment)
				{
					wrp.insert(s, offset,bias);
				}
				//We add the content anyway
				new_content.add(obj);
				to_be_detached.add(obj);
			}
			//Otherwise it is a text element that we can simply mutate
			else if ( obj instanceof XMLText )
			{
				String st=((XMLText)obj).getValue();
				increment = st.length();
				//Here we mutate the content of the XMLText in case the offset
				//is withing its range.
				if (cur_offset<=offset && offset<cur_offset+increment)
				{
					((XMLText)obj).setValue(insertString(st, s, offset-cur_offset, bias));
					//We fire a new document event
					event = new XMLDocument.DocumentEvent(doc, offset, s);
					event.addEdit(new XMLDocument.ElementEdit(((XMLText)obj).getWrapper()));
					event.fire();
				}
				//We add the content anyway
				new_content.add(obj);
			}
			cur_offset += increment;
		}

		//In the case the element was empty
		if ( new_content.size()==0 )
		{
			XMLText new_text = new XMLText(s, element);
			new_content.add(new_text);
			//We create the DocumentEvent associated with the change and fire
			//it to the containing document.
			event = new XMLDocument.DocumentEvent(doc, offset, s);
			XMLDocument.ElementEdit edit = new XMLDocument.ElementEdit(this);
			edit.addAddedElement(new_text.getWrapper());
			event.addEdit(edit);
			event.fire();
		}

		//Now we detach the nodes
		for (Enumeration e=to_be_detached.elements();e.hasMoreElements();)
		{ ((XMLElement)e.nextElement()).detach(); }

		//We set the new content
		element.setNewContent(new_content);
	}

	/**
	  * Inserts the given element at the given offset in the text. The
	  * rules for insertion are exaclty the same as the other
	  * <code>insert</code> method.
	  * @param <code>elt</code> The <code>XMLElement</code> to be inserted.
	  * @param <code>offset</code> The offset of insertion in the current
	  * text. Once again the offset is absolute and not relative to the
	  * current element.
	  * @param <code>bias</code> The bias as specified before.
	*/
	public void insert(XMLElement elt, int offset, Position.Bias bias)
	{
		if ( !contains(offset,offset+1) )
		{	System.out.println("Bad location"); return;	}

		XMLDocument.ElementEdit   edit;
		XMLDocument.DocumentEvent event;
		int			cur_offset = getStartOffset();
		XMLDocument doc = (XMLDocument)element.getDocument();
		ArrayList	new_content = new ArrayList(element.getContent().size()-1);
		Vector		to_be_detached = new Vector();
		String		inserted = elt.getWrapper().getWholeText();

		//We look for the insertion point
		for ( int i=0 ; i<element.getContent().size() ; i++ )
		{
			int increment = 0;

			Object obj = element.getContent().get(i);
			//It is an Element
			if ( obj instanceof XMLElement )
			{
				XMLElementWrapper wrp = ((XMLElement)obj).getWrapper();
				increment = wrp.getLength();
				//If the element contains the offset at which we want to do the insertion
				//then we recurse
				if (cur_offset<=offset && offset<cur_offset+increment)
				{	wrp.insert(elt, offset,bias); }
				new_content.add(obj);
				to_be_detached.add(obj);
			}
			//Otherwise we modify the XMLText
			else if ( obj instanceof XMLText )
			{
				XMLText text = (XMLText)obj;
				String st = text.getValue();
				increment = st.length();
				//The XMLText element contains the insertion point
				if (cur_offset<=offset && offset<cur_offset+increment)
				{
					//str_split it the insertion point in the XMLText content
					int str_split = offset-cur_offset;
					//Do we prepend the insertion?
					if ( bias==Position.Bias.Backward )
					{
						//We split the XMLText content to insert the new element
						text.setValue(st.substring(0,str_split));
						new_content.add(text);
						new_content.add(elt);
						XMLText new_text=new XMLText(st.substring(str_split,st.length()));
						new_content.add(new_text);
						//We fire a new Document event
						event = new XMLDocument.DocumentEvent(doc, offset,
						inserted);
						edit  = new XMLDocument.ElementEdit(this);
						edit.addAddedElement(new_text.getWrapper());
						event.addEdit(edit);
						event.fire();
						System.out.println("Event fire-1");
					}
					//Or do we append it?
					else
					{
						//We check if the offset is the last charchter
						if ( offset+1>=(cur_offset+increment) )
						{
							new_content.add(obj);
							new_content.add(elt);
						}
						//Otherwise we still have to split the current text node
						else
						{
							//We do this because we want to insert AFTER the given offset
							str_split++;
							text.setValue(st.substring(0,str_split));
							new_content.add(text);
							new_content.add(elt);
							XMLText new_text=new XMLText(st.substring(str_split,st.length()));
							new_content.add(new_text);
						}
						//We fire a new Document event
						event = new XMLDocument.DocumentEvent(doc, offset,
						inserted);
						edit  = new XMLDocument.ElementEdit(this);
						edit.addAddedElement(elt.getWrapper());
						event.addEdit(edit);
						event.fire();
						System.out.println("Event fire-2");
					}
				}
				else
				{	new_content.add(obj);	}
			}
			cur_offset += increment;
		}

		//In case the element was empty
		if ( element.getContent().size()==0 )
		{
			new_content.add(elt);
			//We create the DocumentEvent associated with the change and fire
			//it to the containing document.
			event = new XMLDocument.DocumentEvent(doc, offset, inserted);
			edit = new XMLDocument.ElementEdit(this);
			edit.addAddedElement(elt.getWrapper());
			event.addEdit(edit);
			event.fire();
			System.out.println("Event fire-3");
		}

		//Now we detach the nodes
		for (Enumeration e=to_be_detached.elements();e.hasMoreElements();)
		{ ((XMLElement)e.nextElement()).detach(); }

		//We set the new content
		element.setNewContent(new_content);
	}

	/**
	  * Inserts the a string into another at the specified offset. The
	  * bias, as in the other instance method tells if the string has to be
	  * appended or pretended. See the documentation of the instance method
	  * for more information.
	  * @param <code>source</code> is the string in which the other will be
	  * inserted.
	  * @param <code>insert</code> is the string that will be inserted into
	  * the other one.
	  * @param <code>offset</code> is the offset of insertion in the offset
	  * string.
	  * @param <code>bias</code> is backward, this will prepend the string,
	  * if forward it will append it.
	*/
	public static String insertString(String source, String insert, int offset,
	Position.Bias bias)
	{
		if (offset>source.length())
		{	return source;	}

		if ( bias==Position.Bias.Backward )
		{
			return source.substring(0,offset)+insert+
			source.substring(offset,source.length());
		}
		else
		{
			if ( offset+1>=source.length() )
			{
				return source+insert;
			}
			else
			{
				return source.substring(0,offset+1)+insert+
				source.substring(offset+1,source.length());
			}
		}
	}

	/**
	  * Removes the text contained between the start and end offsets, the
	  * end offset being exclusive.<br/>
	  * Elements that are fully contained withing the given range will be
	  * deleted.
	  * @param <code>start</code> The start offset where text deletion will
	  * begin.
	  * @param <code>end</code> The first charachter that will not be
	  * deleted after the start offset.
	  * @return The deleted elements.
	*/
	public Vector remove(int start, int end)
	{
		//We make sure that the offsets are within the element bounds.
		if (!contains(start,end) )
		{ return new Vector(); }

		//We adjust the end
		if ( end>=getEndOffset() )
		{	end = getEndOffset();	}

		ArrayList new_content = new ArrayList(element.getContent().size()-1);
		Vector    to_be_detached = new Vector();
		Vector    result = new Vector();

		//We prepare the event
		XMLDocument doc = (XMLDocument)element.getDocument();
		XMLDocument.DocumentEvent event=new XMLDocument.DocumentEvent(doc, start,
		end-start);
		XMLDocument.ElementEdit edit;

		int offset=getStartOffset();
		for ( int i=0 ; i<element.getContent().size() ; i++ )
		{
			int increment=0;
			Object obj = element.getContent().get(i);
			if ( obj instanceof XMLElement )
			{
				XMLElementWrapper wrp = ((XMLElement)obj).getWrapper();
				increment = wrp.getLength();
				switch ( getOperation(offset, increment, start, end ) )
				{
					case CUT_LEFT:
					case CUT_RIGHT:
					case CUT_INSIDE:
						//In these case we do not have to notify anything
						wrp.remove(start, end);
					case LEAVE:
						new_content.add(wrp.element);
						to_be_detached.add(wrp.element);
						break;
					case DELETE:
						//Here we have to notify of the removal of the given
						//node.
						edit = new XMLDocument.ElementEdit(this);
						edit.addAddedElement(wrp);
						event.fire();
						to_be_detached.add(wrp.element);
						result.add(wrp.element);
				}
			}
			else if (obj instanceof XMLText)
			{
				XMLText text=(XMLText)obj;
				String s = text.getValue();
				increment = s.length();
				s = cutString(s, offset, start, end);
				//The text has been changed
				if ( s!=null )
				{
					text.setValue(s);
					new_content.add(text);
				}
				//The text(and thus the XMLText) has been removed
				else
				{
				}
			}
			offset += increment;
		}

		//We fire the event
		//event.fire();
		//Now we detach the nodes
		for (Enumeration e=to_be_detached.elements();e.hasMoreElements();)
		{ ((XMLElement)e.nextElement()).detach(); }
		element.setNewContent(new_content);
		return result;

	}

	//------------------------------------------------------------------------
	//
	// Event tools
	//
	//------------------------------------------------------------------------

	//------------------------------------------------------------------------
	//
	// String slaughter
	//
	//------------------------------------------------------------------------

	/**
	  * Tells which operation should be made on the segmen represented by
	  * the given offset and the given length, given the fact that the cut
	  * will go from <code>start</code> to <code>end</code> exluded.
	*/
	public static short getOperation(int offset, int length, int start, int end)
	{
		int bstart = offset;
		int bend   = offset+length;

		if ( start>=end || start>=bend || bstart>=end )
		{	return LEAVE;	}

		if (      bstart<start  && bend<=end )
		{	return CUT_RIGHT;	}
		else if ( start<=bstart && end<bend  )
		{	return CUT_LEFT;	}
		else if ( bstart<start  && end<bend  )
		{	return CUT_INSIDE;	}
		else if ( start<=bstart && bend<=end )
		{	return DELETE;	}
		else
		{	return LEAVE;	}
	}

	/**
	  * Cuts the given string according to the operation adviced by the
	  * <code>getOperation</code> method.
	*/
	public static String cutString( String s, int offset, int start, int end)
	{
		switch ( getOperation(offset, s.length(), start, end) )
		{
			case CUT_LEFT:
				return s.substring(end-offset,s.length());
			case CUT_RIGHT:
				return s.substring(0,start-offset);
			case CUT_INSIDE:
				return s.substring(0,start-offset)+s.substring(end-offset,
				s.length());
			case LEAVE:
				return s;
			default:
				return null;
		}
	}
	public String getWholeText()
	{
		return getWholeText(getStartOffset(), getEndOffset());
	}

	/**
	  * This is an extension to the JDOM getText() method. This basically not
	  * only gives the depth-1 contained text, but also includes the text of
	  * Element children. This is actually the approach the JDOM should have
	  * had.<br/>
	  * In case the end offset exceeds the right bound of the wrapped
	  * element text, then it will be set to the actual right bound.
	  * @param <code>start</code> is the start offset in the text.
	  * @param <code>end</code> is the end offset in the text, and is
	  * exclusive, which means if you add <code>getLength()</code> to
	  * <code>getStartOffset()</code>, you will be out of the bounds of the
	  * current element.</code>
	*/
	public String getWholeText(int start, int end)
	{
		//If the offset are completly out of bounds then we return null
		if (end<getStartOffset() && start>=getEndOffset() || start>end)
		{
			return "";
		}

		//If the end offset is too big, then we correct it.
		if (end>=getEndOffset())
		{
			end=getEndOffset();
		}
		if (start<=getStartOffset())
		{
			start=getStartOffset();
		}

		StringBuffer	buffer=new StringBuffer();
		ListIterator iterator = element.getContent().listIterator();

		while( iterator.hasNext() )
		{
			Object elt = iterator.next();
			if ( elt instanceof XMLElement )
			{
				XMLElementWrapper wrapper = ((XMLElement)elt).getWrapper();
				buffer.append(wrapper.getWholeText(wrapper.getStartOffset(),
				end));
			}
			else if ( elt instanceof org.jdom.CDATA )
			{
				String s = ((org.jdom.CDATA)elt).getText();
				//We may have to cut the text
				if ( (start+buffer.length()+s.length()) < end )
				{
					buffer.append(s);
				}
				else
				{
					int lbound = end-(start+buffer.length());
					if (lbound>0)
					{	buffer.append(s.substring(0,lbound));	}
				}
			}
			else if ( elt instanceof XMLText)
			{
				String s = ((XMLText)elt).getValue();
				//We may have to cut the text
				if ( (start+buffer.length()+s.length()) < end )
				{
					buffer.append(s);
				}
				else
				{
					int lbound = end-(start+buffer.length());
					if (lbound>0)
					{	buffer.append(s.substring(0,lbound));	}
				}
			}
			else
			{
				System.err.println("Unknown content: "+elt);
			}
		}
		return buffer.toString();
	}


	//------------------------------------------------------------------------
	//
	// Attributes access
	//
	//------------------------------------------------------------------------


	/**
	  * Fetches the collection of attributes this element contains.
	*/
	public AttributeSet getAttributes()
	{
		return element.xmlAttributes;
	}

	/**
	  * Fetches the document associated with this element.
	*/
	public Document getDocument()
	{
		return (Document)element.getDocument();
	}

	/**
	  * Returns the name of the given text element.
	*/
	public String getName()
	{
		return element.getName();
	}

	/**
	  * Returns the parents Element, which can be null if the Element is a
	  * root.
	*/
	public Element getParentElement() 
	{
		if ( element.getParent()!=null )
		{
			return ((XMLElement)element.getParent()).getWrapper();
		}
		else
		{
			return null;
		}
	}

	//-------------------------------------------------------------------------
	//
	// Miscelaneous
	//
	//-------------------------------------------------------------------------

	/** 
	  * Returns true if the Element has no children, otherwise returns false.
	*/
	public boolean isLeaf()
	{
		if ( getElementCount()>0 )
		{	return false;	}
		else
		{	return true;	}
	}

	/**
	  * Returns the <code>XMLElement</code> that this wrapper wraps.
	*/
	public XMLElement getXMLElement()
	{
		return element;
	}

	// TO_STRING_______________________________________________________________

	public String toString()
	{
		return getName()+"["+getStartOffset()+":"+getEndOffset()+"]";
	}
}

// EOF-Unix/ASCII--------------------------------------@RisingSun//Java/1.0//EN
