// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// ClassName		:	XMLTextWrapper
// Version			:	0.90
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	17-Sep-2001	
// Last mod.		:	15-Oct-2001	
// History			:
//						15-Oct-2001 Added a getWholeText method that handles
//						start and end offsets.(SP)
//						19-Sep-2001 Renamed XMLStringWrapper, now encapsulates
//						a StringBuffer and is aware of changes.
//						17-Sep-2001 First implementation.(SP)
//
// Notes			:
//
// Bugs				:
//					  -
//
// To do			:
//					  -

package net.sourceforge.spedit.core;

import java.util.*;
import javax.swing.text.*;
import javax.swing.tree.TreeNode;
import org.jdom.CDATA;
import org.jdom.Text;


//----------------------------------------------------------------------------
//
//  XMLTextWrapper
//
//----------------------------------------------------------------------------


/**
  * The <code>XMLStringWrapper</code> class wraps an <code>XMLString</code> so
  * as to it becomes usable in the Swing document model. As XMLString are
  * mutable, the wrapper contains a reference to its attached XMLString and
  * vice-versa.
*/
public class XMLTextWrapper implements Element, XMLNode, TreeNode
{

	protected XMLText element;
	protected XMLElement parent;

	//------------------------------------------------------------------------
	//
	// Constructors
	//
	//------------------------------------------------------------------------

	public XMLTextWrapper(XMLText element)
	{
		this.element = element;
	}

	//------------------------------------------------------------------------
	//
	// Element accession
	//
	//------------------------------------------------------------------------


	// ELEMENT_ACCESSION______________________________________________________

	/**
	  * 
	*/
	public int getElementCount() 
	{
		return 0;
	}

	/**
	  *
	*/
	public Element getElement(int index) 
	{ 
		return null;
	}

	/**
	*/  
	public int getElementIndex(int offset) 
	{
		return -1;
	}

	//----------------------------------------------------------------
	//
	// XMLInformation
	//
	//----------------------------------------------------------------

	public XMLInformation getInformation()
	{
		return null;
	}

	public void setInformation(XMLInformation info)
	{}

	public boolean hasInformation()
	{
		return false;
	}


	//------------------------------------------------------------------------
	//
	// Tree node compliance
	//
	//------------------------------------------------------------------------

	public boolean getAllowsChildren() 
	{	return false; }

	public Enumeration children()
	{	return null; }

	public TreeNode getChildAt(int childIndex) 
	{	return null; }

	public int getChildCount() 
	{	return 0; }

	public int getIndex(TreeNode node) 
	{	return -1; }

	public TreeNode getParent() 
	{	return (XMLElementWrapper)getParentElement(); }
	
	//----------------------------------------------------------------
	//
	// Offsets
	//
	//----------------------------------------------------------------

	/**
	  * Fetches the offset from the beginning of the document that this
	  * element begins at.
	*/
	public int getStartOffset() 
	{
		int soffset=((XMLElement)element.getParent()).getWrapper().getStartOffsetOf(element);
		return soffset;
	}

	/**
	  * Fetches the offset from the beginning of the document that this
	  * element ends at.
	*/
	public int getEndOffset() 
	{
		return getStartOffset()+getLength();
	}

	/**
	  * Returns the length of this element, int terms of text.
	  * This is simply the difference between <code>getEndOffset()</code>
	  * and <code>getStartOffset()</code>.
	*/
	public int getLength()
	{
		return element.getValue().length();
	} 

	// ATTRIBUTES_ACCESS______________________________________________

	/**
	  * Fetches the collection of attributes this element contains.
	*/
	public AttributeSet getAttributes()
	{
		//TODO:Fix this
		return null;
	}

	/**
	  * Fetches the document associated with this element.
	*/
	public Document getDocument()
	{
		return (Document)element.getDocument();
	}

	/**
	  * Returns the name of the given text element.
	*/
	public String getName()
	{
		return "Text";
	}

	//TODO:Remove this one
	public String getContent()
	{
		return element.toString();
	}

	public String getWholeText()
	{
		return element.toString();
	}

	public String getWholeText(int start, int end)
	{
		start -= getStartOffset();
		end -= getStartOffset();
		return element.toString().substring(start, end);
	}


	/**
	  * Returns the parents Element, which can be null if the Element is a
	  * root.
	*/
	public Element getParentElement() 
	{
		return ((XMLElement)element.getParent()).getWrapper();
	}

	// MISC___________________________________________________________

	/** 
	  * Always return true as every wrapped Text element can have no children.
	*/
	public boolean isLeaf()
	{
		return true;
	}

	public String toString()
	{
		return "["+getStartOffset()+":"+getEndOffset()+"]"+element.toString();
	}
}

