// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes-TestCases
// ClassName		:	StyleableTestCase
// Version			:	0.08
// Type				:	Test case
//
// ThreadSafe		:	-
// Description		:	A test case for Styleable.Utils class.
// 						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	26-Sep-2001	
// Last mod.		:	26-Sep-2001	
// History			:
//						26-Sep-2001 First implementation.(SP)
//
// Bugs				:
//					  -
//
// To do			:
//					  -
//					  -
//					

package net.sourceforge.spedit.core.test;

import junit.framework.*;
import javax.swing.text.*;
import java.util.*;
import java.awt.Color;
import net.sourceforge.spedit.core.*;

public class StyleableTestCase extends TestCase
{

	public StyleableTestCase(String name)
	{
		super(name);
	}

	protected void setUp()
	{
	}

	//-------------------------------------------------------------------------
	//
	// Test suite
	//
	//-------------------------------------------------------------------------

	/**
	  * Test the recognition of colors
	*/
	public void testIdentifyColor()
	{
		assertEquals(Styleable.Utils.COLOR, Styleable.Utils.identify("#FFFFFF"));
		assertEquals(new Color(255,255,255), Styleable.Utils.parseColor("#FFFFFF"));
		assertEquals(Styleable.Utils.COLOR, Styleable.Utils.identify("#FF201009"));
		assertEquals(new Color(255,32,16,9), Styleable.Utils.parseColor("#FF201009"));
		assertEquals(Styleable.Utils.UNKNOWN, Styleable.Utils.identify("#FFFFF"));
		assertEquals(Styleable.Utils.UNKNOWN, Styleable.Utils.identify("#FFFF"));
	}

	/**
	  * Test the recognition of pixels
	*/
	public void testIdentifyPixel()
	{
		assertEquals(Styleable.Utils.PIXEL, Styleable.Utils.identify("10"));
		assertEquals(10, Styleable.Utils.parsePixel("10"));
		assertEquals(Styleable.Utils.UNKNOWN, Styleable.Utils.identify("10a"));
	}

	/**
	  * Test the recognition of points
	*/
	public void testIdentifyPoint()
	{
		assertEquals(Styleable.Utils.POINT, Styleable.Utils.identify("10pt"));
		assertEquals(Styleable.Utils.POINT, Styleable.Utils.identify("10.1pt"));
		assert(10.0==Styleable.Utils.parsePoint("10.0pt"));
		assert(10.1==Styleable.Utils.parsePoint("10.1pt"));
		assertEquals(Styleable.Utils.UNKNOWN, Styleable.Utils.identify("10.0p"));
	}

	/**
	  * Test the splitStyle method. This tests a simple property, a property
	  * nested in two groups, and a property nested in three groups.
	*/
	public void testSplitStyle()
	{
		String[] st = Styleable.Utils.splitStyle("block");
		assertEquals(1,  st.length);
		assertEquals("block",  st[0]);

		st = Styleable.Utils.splitStyle("block.border");
		assertEquals(2,  st.length);
		assertEquals("block",  st[0]);
		assertEquals("border", st[1]);


		st = Styleable.Utils.splitStyle("block.border.color");
		assertEquals(3,  st.length);
		assertEquals("block",  st[0]);
		assertEquals("border", st[1]);
		assertEquals("color",  st[2]);
	}

	//-------------------------------------------------------------------------
	//
	// Testing the style object
	//
	//-------------------------------------------------------------------------

	/**
	  * Tests the <code>Style</code> object.
	*/
	public void testStyle()
	{
		Styleable.StyleSheet stylea = new Styleable.StyleSheet();
		Styleable.StyleSheet styleb = new Styleable.StyleSheet();

		stylea.setStyleClass("font.name", String.class);
		assertEquals(String.class, stylea.getStyleClass("font.name"));

		stylea.setStyle("font.name", "Times");
		assertEquals("Times", stylea.getStyle("font.name"));

		//We test the inheritance of the fonts
		styleb.inheritStyle("font", stylea);
		assertEquals("Times", styleb.getStyle("font.name"));
		assertEquals(String.class, stylea.getStyleClass("font.name"));
		
		//We check the dynamicity
		stylea.setStyle("font.name", "Arial");
		assertEquals("Arial", stylea.getStyle("font.name"));
		assertEquals("Arial", styleb.getStyle("font.name"));

		//We mask the font.name property in styleb
		styleb.setStyle("font.name", "AvantGarde");
		assertEquals("Arial", stylea.getStyle("font.name"));
		assertEquals("AvantGarde", styleb.getStyle("font.name"));

		styleb.inheritStyle(stylea);
		stylea.setStyle("border.size",new Integer(1));
		assertEquals(new Integer(1), stylea.getStyle("border.size"));
		assertEquals(new Integer(1), styleb.getStyle("border.size"));
	}

	//-------------------------------------------------------------------------
	//
	// Integration
	//
	//-------------------------------------------------------------------------

	public static Test suite()
	{
		TestSuite suite = new TestSuite(StyleableTestCase.class);
		return suite;
	}

	public static void main(String args[])
	{
		String[] testCaseName = {StyleableTestCase.class.getName()};
		junit.swingui.TestRunner.main(testCaseName);
	}
}

// EOF-Unix/ASCII--------------------------------------@RisingSun//Java/1.0//EN
