// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes-TestCases
// ClassName		:	XMLDocumentTestCase
// Version			:	0.40
// Type				:	Test case
//
// ThreadSafe		:	-
// Description		:	A test case for the XMLDocument class.
// 						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	10-Sep-2001	
// Last mod.		:	27-Sep-2001	
// History			:
//						27-Sep-2001 Position test case.(SP)
//						10-Sep-2001 First implementation.(SP)
//
// Bugs				:
//					  -
//
// To do			:
//					  -Test events (seems like difficult).  
//					  -Extensive test of the getText method
//					
//					

package net.sourceforge.spedit.core.test;

import junit.framework.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.util.*;
import net.sourceforge.spedit.core.*;


public class XMLDocumentTestCase extends TestCase
{

	XMLDocument doc;
	DocumentEvent docEvent;

	public XMLDocumentTestCase(String name)
	{
		super(name);
	}

	protected void setUp()
	{
		doc = new XMLDocument();
	}

	//-------------------------------------------------------------------------
	//
	// Test suite
	//
	//-------------------------------------------------------------------------

	/**
	  * The purpose of this test is to check that the properties can be set and
	  * retrieved properly.
	*/
	public void testProperties()
	{
		Vector value = new Vector();
		value.add("Pouet");
		doc.putProperty("Spam", value);
		assert(value==doc.getProperty("Spam"));
		value.add("Poupou");
		assert(value==doc.getProperty("Spam"));
		doc.removeProperty("Spam");
		assert(doc.getProperty("Spam")==null);
	}

	/**
	  * The purpose of this test is to check that when positions are registered
	  * into the document they follow any change made to it.
	*/
	public void testPositions()
	{
		XMLElement  root = new XMLElement("root");
		XMLElement  child= new XMLElement("child");
		XMLDocument doc = new XMLDocument(root);
		root.addContent("01234");
		root.addContent(child);
		child.addContent("56789");
		root.addContent("01234");
		Position start = doc.createPosition(0);
		Position end   = doc.createPosition(14);
		Position cpos  = doc.createPosition(5);

		//We check position update after insertion
		root.getWrapper().insert("popo", 3, Position.Bias.Forward) ;
		assertEquals(0, start.getOffset());
		assertEquals(18, end.getOffset());
		assertEquals(9, cpos.getOffset());

		//We check position update after removal
		root.getWrapper().remove(3,7);
		assertEquals(0, start.getOffset());
		assertEquals(14, end.getOffset());
		assertEquals(5, cpos.getOffset());
	}
	//-------------------------------------------------------------------------
	//
	// Integration
	//
	//-------------------------------------------------------------------------

	public static Test suite()
	{
		TestSuite suite = new TestSuite(XMLDocumentTestCase.class);
		return suite;
	}

	public static void main(String args[])
	{
		String[] testCaseName = {XMLDocumentTestCase.class.getName()};
		junit.swingui.TestRunner.main(testCaseName);
	}
}

// EOF-Unix/ASCII--------------------------------------@RisingSun//Java/1.0//EN
