// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes-TestCases
// ClassName		:	XMLAttributeSetTestCase
// Version			:	0.30
// Type				:	Test case
//
// ThreadSafe		:	-
// Description		:	A test case for the XMLElement and XMLTest classes.
// 						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	10-Sep-2001	
// Last mod.		:	14-Sep-2001	
// History			:
//						14-Sep-2001 Lots of other test case.
//						11-Sep-2001 Fixed small bugs.(SP) 
//						10-Sep-2001 First implementation.(SP)
//
// Bugs				:
//					  -
//
// To do			:
//					  -Test case for testElementInsertion is weak.(SP)
//					

package net.sourceforge.spedit.core.test;

import java.util.*;
import junit.framework.*;
import javax.swing.text.*;
import net.sourceforge.spedit.core.*;

public class XMLElementTestCase extends TestCase
{
	private XMLDocument doc;
	private XMLElement  docroot;

	public XMLElementTestCase(String name)
	{
		super(name);
	}

	protected void setUp()
	{
		doc = new XMLDocument(new XMLElement("root"));
		docroot = (XMLElement)doc.getRootElement();
	}

	protected void tearDown()
	{
		
	}

	//-------------------------------------------------------------------------
	//
	// Tools testing
	//
	//-------------------------------------------------------------------------

	public void testGetOperation()
	{
		XMLElementWrapper wrp = new XMLElement().getWrapper();

		//We test the left cut
		assertEquals(wrp.CUT_LEFT,wrp.getOperation(10,10,5,15));
		assertEquals(wrp.CUT_LEFT,wrp.getOperation(10,10,10,15));
		assertEquals(wrp.CUT_LEFT,wrp.getOperation(10,10,10,19));
		//We test the right cut
		assertEquals(wrp.CUT_RIGHT,wrp.getOperation(10,10,15,25));
		assertEquals(wrp.CUT_RIGHT,wrp.getOperation(10,10,15,20));
		assertEquals(wrp.CUT_RIGHT, wrp.getOperation(10,10,11,20));
		//We test the inside cut
		assertEquals(wrp.CUT_INSIDE, wrp.getOperation(10,10,11,19));
		//We test the deletion
		assertEquals(wrp.DELETE, wrp.getOperation(10,10,10,20));
		assertEquals(wrp.DELETE, wrp.getOperation(10,10,9,20));
		assertEquals(wrp.DELETE, wrp.getOperation(10,10,9,21));
		//We test the leave
		assertEquals(wrp.LEAVE, wrp.getOperation(10,10,0,10));
		assertEquals(wrp.LEAVE, wrp.getOperation(10,10,0,9));
		assertEquals(wrp.LEAVE, wrp.getOperation(10,10,20,22));
	}

	public void testCutString()
	{
		XMLElementWrapper wrp = new XMLElement().getWrapper();
		String s = new String("ABCDEF");

		//We test the left cut
		assertEquals("BCDEF", wrp.cutString(s,3,0,4));
		assertEquals("BCDEF", wrp.cutString(s,3,2,4));
		assertEquals("EF", wrp.cutString(s,3,3,7));
		assertEquals("F", wrp.cutString(s,3,3,8));
		//We test the right cut
		assertEquals("ABCDE",wrp.cutString(s,3,8,10));
		assertEquals("A",wrp.cutString(s,3,4,10));
		assertEquals("ABCDE",wrp.cutString(s,3,8,9));
		//We test the inside cut
		assertEquals("ABF",wrp.cutString(s,3,5,8));
		assertEquals("ABF",wrp.cutString(s,3,5,8));
		assertEquals("ACDEF",wrp.cutString(s,3,4,5));
		//We test the deletion
		assert(wrp.cutString(s,3,3,9)==null);
	}

	public void testStaticInsert()
	{
		String source="ABCDE";
		XMLElementWrapper wrp = new XMLElement().getWrapper();

		assertEquals("XABCDE", wrp.insertString(source,"X",0,Position.Bias.Backward));
		assertEquals("AXBCDE", wrp.insertString(source,"X",0,Position.Bias.Forward));
		assertEquals("ABCDXE", wrp.insertString(source,"X",4,Position.Bias.Backward));
		assertEquals("ABCDEX", wrp.insertString(source,"X",4,Position.Bias.Forward));
		assertEquals("ABXCDE", wrp.insertString(source,"X",2,Position.Bias.Backward));
		assertEquals("ABXCDE", wrp.insertString(source,"X",1,Position.Bias.Forward));
	}

	//-------------------------------------------------------------------------
	//
	// XMLElement test
	//
	//-------------------------------------------------------------------------

	public void testElementCreation()
	{
		//We check the values of the element at construction
		XMLElement elem = new XMLElement("Pouet");
		assert(elem.getDocument()==null);
		assert(elem.getParent()==null);
		assertEquals(elem.getName(), "Pouet");

		//Then we add a child
		XMLElement elem2 = new XMLElement("Pouet-2");
		elem.addContent(elem2);

		//And make sure the axioms are verified
		assert(elem.getWrapper().getElementCount()==1);
		assert(elem.getWrapper().getElement(0)==elem2.getWrapper());
		assert(elem2.getParent()==elem);
		assert(elem2.getDocument()==null);
		
		//We check that the wrapping count is different from the core element
		//count
		elem.addContent("qweqwqe");
		assert(elem.getWrapper().getElementCount()==2);
		assert(elem.getWrapper().getElement(0)==elem2.getWrapper());

		//Do the same for elem2
		elem2.addContent("qweqwqe");
		assert(elem2.getWrapper().getElementCount()==1);

		XMLElement pouet = new XMLElement();
		elem2.addContent(pouet);
		assert(elem2.getWrapper().getElementCount()==2);
		assertEquals(pouet.getWrapper(), elem2.getWrapper().getElement(1));

		//Then we change the parent document, and check that the
		//child has also changed of document
		elem.setDocument(doc);
		assert(elem.getDocument()!=null);
		assert(elem2.getDocument()==elem.getDocument());
		
		//Then we remove the child and verify the axioms
		elem.removeContent(elem2);
		assert(elem2.getParent()!=elem);
		assert(elem.getWrapper().getElementCount()==1);
	}

	public void testElementInsertion()
	{
		XMLElement elem  = new XMLElement("Pouet1");
		XMLElement elem2 = new XMLElement("Spam1");

		//We add the elements to the document
		docroot.addContent(elem);

		//We add a structured content
		elem.addContent("123890");
		elem2.addContent("4567");

		elem.getWrapper().insert(elem2,3,Position.Bias.Backward);
		assertEquals("1234567890", elem.getWrapper().getWholeText());
		assertEquals(elem2.getWrapper(), elem.getWrapper().getElement(1));

		//We add the elements to the document
		docroot.removeContent(elem);
		docroot.removeContent(elem2);
	}

	public void testElementWrapper()
	{
		XMLElement elem  = new XMLElement("Pouet2");
		XMLElement elem2 = new XMLElement("Spam2");
		XMLElement elem3 = new XMLElement("Foobar2");

		//We add a structured content
		elem.addContent("12345");
		elem.addContent(elem2);
		elem.addContent(elem3);
		elem2.addContent("6789");
		elem3.addContent("0123");
		elem.addContent("45");

		//Verify the attribute axioms
		assert(elem.getWrapper().getElementCount()==4);
		assert(elem.getWrapper().getElement(1)==elem2.getWrapper());
		assert(elem.getWrapper().getStartOffset()==0);
		assert(elem.getWrapper().getLength()==15);
		assert(elem.getWrapper().getEndOffset()==15);
		assertEquals(5, elem2.getWrapper().getStartOffset());
		assertEquals(4, elem2.getWrapper().getLength());
		assertEquals(9, elem2.getWrapper().getEndOffset());

		//And verify that the wrapper works well
		assertEquals("123456789012345", elem.getWrapper().getWholeText());
		assertEquals( 0, elem.getWrapper().getElementIndex(-1));
		assertEquals( 0, elem.getWrapper().getElementIndex(4));
		assertEquals( 1, elem.getWrapper().getElementIndex(5));
		assertEquals( 1, elem.getWrapper().getElementIndex(8));
		assertEquals( 2, elem.getWrapper().getElementIndex(9));
		assertEquals( 2, elem.getWrapper().getElementIndex(10));
		assertEquals( 2, elem.getWrapper().getElementIndex(12));
		assertEquals( 3, elem.getWrapper().getElementIndex(13));
		assertEquals( 3, elem.getWrapper().getElementIndex(14));
		assertEquals( 3, elem.getWrapper().getElementIndex(15));
	}

	public void testElementOffsets()
	{
		XMLElement elem  = new XMLElement("Pouet");
		XMLElement elem2 = new XMLElement("Spam");
		XMLElement elem3 = new XMLElement("Spam");

		elem.setDocument(doc);
		elem.addContent("012345");
		elem2.addContent("6789");
		elem3.addContent("01");
		elem.addContent(elem2);

		//We check that a child related to its parent
		assert(elem.getWrapper().getStartOffset()==0);
		assert(elem.getWrapper().getEndOffset()==10);
		assert(elem2.getWrapper().getStartOffset()==6);
		assert(elem2.getWrapper().getEndOffset()==10);

		//Even when the parent is moved to another parent
		elem2.addContent(elem3);
		assert(elem.getWrapper().getStartOffset()==0);
		assert(elem.getWrapper().getEndOffset()==12);
		assert(elem2.getWrapper().getStartOffset()==6);
		assert(elem2.getWrapper().getEndOffset()==12);
		assert(elem3.getWrapper().getStartOffset()==10);
		assert(elem3.getWrapper().getEndOffset()==12);
	}

	public void testElementRange()
	{
		XMLElement elem  = new XMLElement("Pouet");
		XMLElement elem2 = new XMLElement("Spam");

		//We add a structured content
		elem.addContent("12345");
		elem.addContent(elem2);
		elem2.addContent("6789");
		elem.addContent("012345");

		//We check that the range work well
		assert(elem.getWrapper().contains(0,15));
		assert(elem.getWrapper().contains(0,16));
		assert(elem.getWrapper().contains(14,16));
		assert(!elem.getWrapper().contains(15,16));
		assert(elem2.getWrapper().contains(0,15));
		assert(!elem2.getWrapper().contains(0,4));
		assert(elem2.getWrapper().contains(0,5));
		assert(elem2.getWrapper().contains(5,15));
		assert(elem2.getWrapper().contains(5,8));
		assert(elem2.getWrapper().contains(5,9));
		assert(!elem2.getWrapper().contains(9,10));
	}

	/**
	  * Tests the following aspects:
	  * <ul>
	  *   <li>Removal of a part of a text node.</li>
	  *   <li>Removal of an element inside an element</li>
	  *   <li>Coherence of the node content during and after the removals.</li>
	  * </ul>
	*/
	public void testElementTextRemoval()
	{
		XMLElement elem  = new XMLElement("Pouet");
		XMLElement elem2 = new XMLElement("Spam");

		//We add a structured content
		elem.addContent("12345");
		elem.addContent(elem2);
		elem2.addContent("6789");
		elem.addContent("ABCDE");

		//We check that the range work well
		elem.getWrapper().remove(10,15);
		assertEquals("123456789A",elem.getWrapper().getWholeText());
		assert(elem.getWrapper().getElementCount()==3);
		assert(elem.getWrapper().getElement(1)==elem2.getWrapper());

		//And we remove an element
		elem.getWrapper().remove(elem2.getWrapper().getStartOffset(),elem2.getWrapper().getEndOffset());
		assert(elem2.getParent()==null);
		assertEquals("12345A",elem.getWrapper().getWholeText());
	}

	/**
	  * Test the coeherence of an element after adding text in it.
	*/
	public void testElementTextInsertion()
	{
		XMLElement elem  = new XMLElement("Pouet");
		XMLElement elem2 = new XMLElement("Spam");

		//We add a structured content
		elem.addContent("12345");
		elem.addContent(elem2);
		elem2.addContent("6789");
		elem.addContent("ABCDE");

		//We have text content "123456789ABCDE"
		//with offsets          01234567890123"
		//                      00000000001111
		elem.getWrapper().insert("XX",3,Position.Bias.Forward);
		assertEquals("1234XX56789ABCDE", elem.getWrapper().getWholeText());

		elem.getWrapper().insert("YY",7,Position.Bias.Backward);
		assertEquals("1234XX5YY6789ABCDE", elem.getWrapper().getWholeText());
		assertEquals("YY6789", elem2.getWrapper().getWholeText());
	}

	//-------------------------------------------------------------------------
	//
	// XMLAttributes test
	//
	//-------------------------------------------------------------------------

	public void testSetAttribute()
	{
		XMLElement elem = new XMLElement("Pouet");

		String sampleKey   = "Spam";
		String sampleValue = "Foobar";

		elem.setAttribute(sampleKey, sampleValue);
		AttributeSet attrs = elem.getWrapper().getAttributes();

		//We check what happens when we add the attribute.
		assert(attrs.isDefined(sampleKey));
		assert(attrs.containsAttribute(sampleKey, sampleValue));
		assert(attrs.getAttributeCount()==1);
		assertEquals( sampleValue, attrs.getAttribute(sampleKey) );
		assert(attrs.containsAttributes(attrs));

	}

	public void testRemoveAttribute()
	{
		XMLElement elem = new XMLElement("Pouet");

		String sampleKey   = "Spam";
		String sampleValue = "Foobar";

		elem.setAttribute(sampleKey, sampleValue);
		AttributeSet attrs = elem.getWrapper().getAttributes();
		assert(attrs.containsAttribute(sampleKey, sampleValue));
		assert(attrs.getAttributeCount()==1);

		//And what happens when we remove it.
		elem.removeAttribute(sampleKey);
		assert(attrs.getAttributeCount()==0);
		assert(!attrs.containsAttribute(sampleKey, sampleValue));
		assert(!attrs.isDefined(sampleKey));
		assert(attrs.containsAttributes(attrs));
	}

	public void testClone()
	{
		XMLElement elem = new XMLElement("Pouet");
		String sampleKey   = "Spam";
		String spam = "pouet";

		elem.setAttribute(sampleKey, spam);
		AttributeSet attrs = elem.getWrapper().getAttributes();

		AttributeSet attrs2 = attrs.copyAttributes();
		AttributeSet attrs3 = attrs.copyAttributes();

		assert(attrs2.containsAttributes(attrs));
		assert(attrs3.containsAttributes(attrs));

		//We change the attr value, and check that attrs2 has not changed.
		elem.setAttribute(sampleKey, spam+"c");
		assert(!attrs.containsAttributes(attrs2));
		assert(attrs3.containsAttributes(attrs2));
	}

	//-------------------------------------------------------------------------
	//
	// Integration
	//
	//-------------------------------------------------------------------------

	public static Test suite()
	{
		TestSuite suite = new TestSuite(XMLElementTestCase.class);
		return suite;
	}

	public static void main(String args[])
	{
		String[] testCaseName = {XMLElementTestCase.class.getName()};
		junit.swingui.TestRunner.main(testCaseName);
	}
}

// EOF-Unix/ASCII-------------------------------------@RisingSun//Java//1.0//EN
