// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes-TestCases
// ClassName		:	XMLTextTestCase
// Version			:	0.08
// Type				:	Test case
//
// ThreadSafe		:	-
// Description		:	A test case for the XMLText and XMLTextWrapper classes.
// 						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	20-Sep-2001	
// Last mod.		:	20-Sep-2001	
// History			:
//						-Sep-2001 First implementation.(SP)
//
// Bugs				:
//					  -
//
// To do			:
//					  -
//					  -
//					

package net.sourceforge.spedit.core.test;

import junit.framework.*;
import javax.swing.text.*;
import java.util.*;
import net.sourceforge.spedit.core.*;


public class XMLTextTestCase extends TestCase
{

	public XMLTextTestCase(String name)
	{
		super(name);
	}

	protected void setUp()
	{
	}

	//-------------------------------------------------------------------------
	//
	// Test suite
	//
	//-------------------------------------------------------------------------

	/**
	  * The purpose of this test is to check that:
	  * <ul>
	  *  <li>When the text content of an XMLElement is changed, the given
	  *  XMLText is changed too.</li>
	  *  <li>When test is added after an XMLText node then the XMLText node is
	  *  extended with the given text</li>
	  *  <li>Same test when appending in the middle, and appending at the
	  *  beginning.</li>
	  * </ul>
	*/
	public void testDynamicity()
	{
		XMLElement elem1 = new XMLElement("Spam");
		XMLElement elem2 = new XMLElement("Foo");
		XMLText    text1 = new XMLText("1234");
		XMLText    text2 = new XMLText("9012");

		//We create the test case data
		elem1.addContent(text1);
		elem1.addContent(elem2);
		elem2.addContent("popo");
		elem1.addContent(text2);

		assertEquals("1234", text1.getValue());
		assertEquals("9012", text2.getValue());

		//We check that the offsets are OK
		assertEquals(0, text1.getWrapper().getStartOffset());
		assertEquals(4, text1.getWrapper().getEndOffset());
		assertEquals(4, text1.getWrapper().getLength());
		assertEquals(8, text2.getWrapper().getStartOffset());
		assertEquals(12,text2.getWrapper().getEndOffset());
		assertEquals(4, text2.getWrapper().getLength());

		//We check that the text has been dynamically changed when prepending
		elem1.getWrapper().insert("AB", 0, Position.Bias.Backward);
		assertEquals("AB1234", text1.getValue());
		assertEquals(0, text1.getWrapper().getStartOffset());
		assertEquals(6, text1.getWrapper().getEndOffset());
		assertEquals(6, text1.getWrapper().getLength());

		assertEquals(10, text2.getWrapper().getStartOffset());
		assertEquals(14, text2.getWrapper().getEndOffset());
		assertEquals(4, text2.getWrapper().getLength());

		//That it has been changed when appending
		elem1.getWrapper().insert("XY", 13, Position.Bias.Forward);
		assertEquals("9012XY", text2.getValue());
		assertEquals(10, text2.getWrapper().getStartOffset());
		assertEquals(16, text2.getWrapper().getEndOffset());
		assertEquals(6, text2.getWrapper().getLength());

		//That is has been changed when inserting
		elem1.getWrapper().insert("OP", 11, Position.Bias.Forward);
		assertEquals("90OP12XY", text2.getValue());
		assertEquals(10, text2.getWrapper().getStartOffset());
		assertEquals(18, text2.getWrapper().getEndOffset());
		assertEquals(8, text2.getWrapper().getLength());
	}

	public void testStripping()
	{
		XMLText    text1 = new XMLText("1234");
		XMLText    text2 = new XMLText(" 1234");
		XMLText    text3 = new XMLText("  1234");
		XMLText    text4 = new XMLText(" 1234 ");
		XMLText    text5 = new XMLText("  ");
		XMLText    text6 = new XMLText(" ");
		XMLText    text7 = new XMLText("");
		
		text1.strip();
		text2.strip();
		text3.strip();
		text4.strip();
		text5.strip();
		text6.strip();
		text7.strip();
		
		assertEquals("1234",text1.getValue());
		assertEquals("1234",text2.getValue());
		assertEquals("1234",text3.getValue());
		assertEquals("1234",text4.getValue());
		assertEquals("",text5.getValue());
		assertEquals("",text6.getValue());
		assertEquals("",text7.getValue());
	}
	//-------------------------------------------------------------------------
	//
	// Integration
	//
	//-------------------------------------------------------------------------

	public static Test suite()
	{
		TestSuite suite = new TestSuite(XMLTextTestCase.class);
		return suite;
	}

	public static void main(String args[])
	{
		String[] testCaseName = {XMLTextTestCase.class.getName()};
		junit.swingui.TestRunner.main(testCaseName);
	}
}

// EOF-Unix/ASCII--------------------------------------@RisingSun//Java/1.0//EN
