// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// Module			:	GUI-Views
// ClassName		:	AttributesDisplay
// Version			:	0.50
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	Displays the attributes
//						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sbastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	13-Nov-2001	
// Last mod.		:	13-Nov-2001	
// History			:
//						13-Nov-2001 First implementation.(sp)
//
// Notes			:
//
// Bugs				:
//					  -
//
// To do			:
//					  - 

package net.sourceforge.spedit.ui;

import javax.swing.*;
import javax.swing.table.*;
import net.sourceforge.spedit.core.*;

/**
  * <p>The <code>AttributesDisplay</code> is a swing component that is integrated
  * into <code>WrapperView</code> to display the attributes of an xml element.</p>
  *
  * <p>To customize the display of attributes for a specific view, simply
  * subclass <code>AttributesDisplay</code> and adapt it to your needs. By default
  * it will display a table with the different attributes.</p>
*/
public class AttributesDisplay extends JPanel
{
	// PROTECTED_______________________________________________________________
	
	protected JTable			table;
	protected AttributesModel	model;
	
	//-------------------------------------------------------------------------
	//
	// Constructors
	//
	//-------------------------------------------------------------------------

	/**
	  * Creates a new <code>AttributesDisplay</code> that displays the
	  * attributes of the given element.
	*/
	public AttributesDisplay(XMLElement element)
	{
		super();
		this.model = new AttributesModel(element);
		this.table = new JTable(model);
		add(table);
	}
	
	//-------------------------------------------------------------------------
	//
	// Attributes table model
	//
	//-------------------------------------------------------------------------
	
	/**
	  * <p>The <code>AttributesModel</code> represents the attributes bound
	  * to a specific element as a table model.
	*/
	public class AttributesModel extends AbstractTableModel
	{
		protected XMLElement element;
		
		/**
		  * Creates a new AttributesModel that represents as a table
		  * the attributes bound to the given element.
		  * @param element The element from which the attributes will
		  *        be represented.
		*/
		public AttributesModel(XMLElement element)
		{
			this.element = element;
		}
		
		/**
		  * There are always two columns: the first one is the attribute name,
		  * the second one is the attribute value.
		  * @return 2
		*/
		public int getColumnCount()
		{	return 2;	}
		
		/**
		  * Returns the name of the givem column.
		  * @param col the column number, either 0 or 1.
		  * @return <code>Name</code> if col==0, otherwise returns
		  *         <code>Value</code>
		*/
		public String getColumnName(int col)
		{
			if ( col==0 )
			{	return "Name";	}
			else
			{	return "Value";	}
		}
		
		/**
		  * The number of rows is equal to the number of attributes in the
		  * bound <code>XMLElement</code>.
		  * @return the number of attributes contained in the given element.
		*/
		public int getRowCount()
		{	return element.getAttributes().size();	}
		
		/**
		  * Returns either the name(0) or the value(1) of the attribute
		  * at the column given by the <code>rowIndex</code> argument.
		*/
		public Object getValueAt(int rowIndex, int columnIndex)
		{
			if ( columnIndex==0 )
			{
				return ((org.jdom.Attribute)element.getAttributes().get(rowIndex)).getName();
			}
			else
			{
				return ((org.jdom.Attribute)element.getAttributes().get(rowIndex)).getValue();
			}
		}

	}
}

// EOF-Unix/ASCII-------------------------------------@RisingSun//Java//1.0//EN
