// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// Module			:	GUI-Views
// ClassName		:	BlockView
// Version			:	0.08
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	A BoxView wrapper.
//						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sebastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	27-Sep-2001	
// Last mod.		:	27-Sep-2001	
// History			:
//						27-Sep-2001 First implementation. (SP)
//
// Notes			:
//
// Bugs				:
//					  -
//
// To do			:
//					  - Use CSS

package net.sourceforge.spedit.ui;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.text.*;
import javax.swing.text.*;
import javax.swing.event.*;
import net.sourceforge.spedit.core.*;

//-----------------------------------------------------------------------------
//
// BlockView class
//
//-----------------------------------------------------------------------------

/**
  * 
*/  
public class BlockView extends BoxView implements Styleable
{
	// PUBLIC__________________________________________________________________
	public static Integer X_AXIS=new Integer(View.X_AXIS);
	public static Integer Y_AXIS=new Integer(View.Y_AXIS);

	// PROTECTED_______________________________________________________________
	protected Styleable.StyleSheet styleSheet;
	protected boolean focused  = false;
	protected boolean selected = false;

	// CONSTRUCTORS____________________________________________________________
	public BlockView(Element e, Integer axis)
	{
		super(e, axis.intValue());
		styleSheet = new Styleable.StyleSheet("BlockView-"+e.getName(), this);
	}

	//-------------------------------------------------------------------------
	//
	// Painting
	//
	//-------------------------------------------------------------------------

	public void paint(Graphics g, Shape bounds)
	{
		int off = getStartOffset();
		super.paint(g,bounds);
	}

	public void childAllocation(int off, Rectangle r)
	{
		super.childAllocation(off,r);
	}

	//-------------------------------------------------------------------------
	//
	// Model-View
	//
	//-------------------------------------------------------------------------

	/**
	  * Transforms the given shape that corresponds to the container view to
	  * the destination view.
	*/
	protected Shape adjustC2V(Shape shape)
	{
		Rectangle2D bounds = (Rectangle2D)shape.getBounds();
		double x = bounds.getX()+styleSheet.paddingLeft;
		double y = bounds.getY()+styleSheet.paddingTop;
		double w = bounds.getWidth();
		double h = bounds.getHeight();
		bounds.setRect(x,y,w,h);
		return bounds;
	}

	/**
	  * Transforms the given shape that corresponds to the container view to
	  * the destination view.
	*/
	protected Shape adjustV2C(Shape shape)
	{
		Rectangle2D bounds = (Rectangle2D)shape.getBounds();
		double x = bounds.getX()-styleSheet.paddingLeft;
		double y = bounds.getY()-styleSheet.paddingTop;
		double w = bounds.getWidth();
		double h = bounds.getHeight();
		bounds.setRect(x,y,w,h);
		return bounds;
	}
	public int viewToModel(float x, float y, Shape a) 
	{	return super.viewToModel(x-styleSheet.paddingLeft,y-styleSheet.paddingTop,a);	}

	public int viewToModel(float x, float y, Shape a, Position.Bias[] biasReturn) 
	{	return super.viewToModel(x-styleSheet.paddingLeft,y-styleSheet.paddingTop,a,biasReturn);	}

	public Shape modelToView(int p0, Position.Bias b0, int p1, Position.Bias b1, Shape a) 
	throws BadLocationException
	{
		Rectangle2D shape = (Rectangle2D) super.modelToView(p0,b0,p1,b1,a);
		return adjustC2V(shape);
	}

	public Shape modelToView(int pos, Shape a) 
	throws BadLocationException
	{	
		Rectangle2D shape = (Rectangle2D) super.modelToView(pos,a);
		return adjustC2V(shape);
	}

	public Shape modelToView(int pos, Shape a, Position.Bias b) 
	throws BadLocationException
	{	
		Rectangle2D shape = (Rectangle2D) super.modelToView(pos,a,b);
		return adjustC2V(shape);
	}
	//-------------------------------------------------------------------------
	//
	// Dimensions
	//
	//-------------------------------------------------------------------------

	public short getLeftInset()
	{	return styleSheet.paddingLeft; }

	public short getTopInset()
	{	return styleSheet.paddingTop; }

	public short getBottomInset()
	{	return styleSheet.paddingBottom; }

	public short getRightInset()
	{	return styleSheet.paddingRight; }

	public float getAlignment(int axis)
	{
		if ( axis==View.X_AXIS )
		{ return styleSheet.align; }
		else
		{ return super.getAlignment(axis);	}
	}

	//-------------------------------------------------------------------------
	//
	// StyleSheet
	//
	//-------------------------------------------------------------------------

	public Styleable.StyleSheet getStyleSheet()
	{
		return styleSheet;
	}

	public void setStyleSheet(Styleable.StyleSheet s)
	{
		styleSheet=s;
	}

	public Styleable.StyleSheet getParentStyleSheet()
	{
		View parent = getParent();
		if ( parent!=null )
		{
			//We try the parent
			if ( parent instanceof Styleable )
			{
				//We try to get the parent stylesheet.
				Styleable.StyleSheet ss = ((Styleable)parent).getStyleSheet();
				//If there is none we try the grandparent one.
				if ( ss==null )
				{	return  ((Styleable)parent).getParentStyleSheet(); }
				else
				{	return ss;	}
			}
			//Or the grandparent (useful with ParagraphView).
			else if ( (parent=parent.getParent())!=null && parent instanceof
			Styleable )
			{
				 Styleable.StyleSheet ss = ((Styleable)parent).getStyleSheet();
				if ( ss==null )
				{	return  ((Styleable)parent).getParentStyleSheet(); }
				else
				{	return ss;	}
			}
			else
			{	return null;	}
		}
		else
		{
			return null;
		}
	}
}
// EOF-Linux/ISO-8859-1-------------------------------@RisingSun//Java//1.0//EN
