// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// Module			:	GUI-Views
// ClassName		:	BlockView
// Version			:	0.08
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	A BoxView wrapper.
//						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sbastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	27-Sep-2001	
// Last mod.		:	07-Nov-2001	
// History			:
//						07-Nov-2001 Fixed selcetion problem. The solution was
//						to copy the sourcecode from the JDK 1.3.1 and modify
//						the "incriminated" parts.(sp)
//						27-Sep-2001 First implementation. (SP)
//
// Notes			:
//						- Borrows code from the GlyphView JDK 1.3.1 source.
// Bugs				:
//					  -
//
// To do			:
//					  -

package net.sourceforge.spedit.ui;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.text.*;
import javax.swing.text.*;
import javax.swing.event.*;
import net.sourceforge.spedit.core.*;

//-----------------------------------------------------------------------------
//
// BlockView class
//
//-----------------------------------------------------------------------------

public class LabelView extends javax.swing.text.GlyphView implements Styleable
{
	// PUBLIC__________________________________________________________________


	// PROTECTED_______________________________________________________________
	protected Styleable.StyleSheet styleSheet;
	protected Vector		lines;
	protected Position		startPos;
	protected Position		endPos;
	protected boolean focused  = false;
	protected boolean selected = false;

	/**
	  * Constructs a new view wrapped on an element.
	  * @param elem the element
	*/
	public LabelView(Element elem)
	{
		super(elem);
		styleSheet = new StyleSheet("LabelView-"+getElement().getName(), this);
	}

	//-------------------------------------------------------------------------
	//
	// Painting
	//
	//-------------------------------------------------------------------------

	public void paint(Graphics g, Shape bounds)
	{
		//int off = getStartOffset();
		//super.paint(g, adjustC2V(bounds));
		//NOTE: This block is a copy & paste version of the JDK 1.3.1
		//GlyphView sourcecode. This has been made because those classes
		//are too protected. I just hope it doesn't break the license.
		checkPainter();

		boolean paintedText = false;
		Component c = getContainer();
		int p0 = getStartOffset();
		int p1 = getEndOffset();
		Rectangle alloc = (bounds instanceof Rectangle) ? (Rectangle)bounds : bounds.getBounds();
		Shape adjusted = adjustC2V(bounds);
		Color bg = getBackground();
		Color fg = getForeground();
		if (bg != null)
		{
			g.setColor(bg);
			g.fillRect(alloc.x, alloc.y, alloc.width, alloc.height);
		}
		if (c instanceof JTextComponent)
		{
			JTextComponent tc = (JTextComponent) c;
			Highlighter h = tc.getHighlighter();
			if (h instanceof LayeredHighlighter)
			{
				((LayeredHighlighter)h).paintLayeredHighlights
				(g, p0, p1, adjustV2C(bounds), tc, this);
			}
		}
	
		/*if (Utilities.isComposedTextElement(getElement()))
		{
			Utilities.paintComposedText(g, bounds.getBounds(), this);
			paintedText = true;
		}*/
		else if(c instanceof JTextComponent)
		{
			JTextComponent tc = (JTextComponent) c;
			Color selFG = tc.getSelectedTextColor();
			Caret caret = tc.getCaret();
			if ((caret != null) && (! caret.isSelectionVisible()))
			{
				// selection currently not visible
				selFG = fg;
			}
	
			if(selFG != null && !selFG.equals(fg))
			{
				int selStart = tc.getSelectionStart();
				int selEnd = tc.getSelectionEnd();
	
				if(selStart != selEnd)
				{
					// Something is selected, does p0 - p1 fall in that range?
					int pMin;
					int pMax;
	
					if(selStart <= p0)
					{	pMin = p0; }
					else
					{	pMin = Math.min(selStart, p1); }
					
					if(selEnd >= p1)
					{	pMax = p1;	}
					else
					{	pMax = Math.max(selEnd, p0); }
					
					// If pMin == pMax (also == p0), selection isn't in this
					// block.
					if(pMin != pMax)
					{
						paintedText = true;
						if(pMin > p0) 
						{	paintTextUsingColor2(g, adjusted, fg, p0, pMin); }
						paintTextUsingColor2(g, adjusted, selFG, pMin, pMax);
						if(pMax < p1)
						{	paintTextUsingColor2(g, adjusted, fg, pMax, p1); }
					}
				}
			}
		}
		if(!paintedText)
		{	paintTextUsingColor2(g, adjusted, fg, p0, p1); }
	}

	protected void paintTextUsingColor2(Graphics g, Shape a, Color c, int p0, int p1)
	{
		// render the glyphs
		g.setColor(c);
		getGlyphPainter().paint(this, g, a, p0, p1);
	}
	
	//-------------------------------------------------------------------------
	//
	// Model-View
	//
	//-------------------------------------------------------------------------

	/**
	  * Transforms the given shape that corresponds to the container view to
	  * the destination view.
	*/
	protected Shape adjustC2V(Shape shape)
	{
		Rectangle2D bounds = (Rectangle2D)shape.getBounds();
		double x = bounds.getX()+styleSheet.paddingLeft;
		double y = bounds.getY()+styleSheet.paddingTop;
		double w = bounds.getWidth();
		double h = bounds.getHeight();
		bounds.setRect(x,y,w,h);
		return bounds;
	}

	/**
	  * Transforms the given shape that corresponds to the container view to
	  * the destination view.
	*/
	protected Shape adjustV2C(Shape shape)
	{
		Rectangle2D bounds = (Rectangle2D)shape.getBounds();
		double x = bounds.getX()-styleSheet.paddingLeft;
		double y = bounds.getY()-styleSheet.paddingTop;
		double w = bounds.getWidth();
		double h = bounds.getHeight();
		bounds.setRect(x,y,w,h);
		return bounds;
	}
	public int viewToModel(float x, float y, Shape a) 
	{	return super.viewToModel(x-styleSheet.paddingLeft,y-styleSheet.paddingTop,a);	}

	public int viewToModel(float x, float y, Shape a, Position.Bias[] biasReturn) 
	{	return super.viewToModel(x-styleSheet.paddingLeft,y-styleSheet.paddingTop,a,biasReturn);	}

	public Shape modelToView(int p0, Position.Bias b0, int p1, Position.Bias b1, Shape a) 
	throws BadLocationException
	{
		Rectangle2D shape = (Rectangle2D) super.modelToView(p0,b0,p1,b1,a);
		return adjustC2V(shape);
	}

	public Shape modelToView(int pos, Shape a) 
	throws BadLocationException
	{	
		Rectangle2D shape = (Rectangle2D) super.modelToView(pos,a);
		return adjustC2V(shape);
	}

	public Shape modelToView(int pos, Shape a, Position.Bias b) 
	throws BadLocationException
	{	
		Rectangle2D shape = (Rectangle2D) super.modelToView(pos,a,b);
		return adjustC2V(shape);
	}

	/**
	* Fetch the background color to use to render the
	* glyphs.  If there is no background color, null should
	* be returned.  This is implemented to return a cached
	* background color.
	*/
	public Color getBackground()
	{
		return styleSheet.background;
	}

	/**
	* Fetch the foreground color to use to render the
	* glyphs.  If there is no foreground color, null should
	* be returned.  This is implemented to return a cached
	* foreground color.
	*/
	public Color getForeground()
	{
		return styleSheet.foreground;
	}

	/**
	  * Fetch the font that the glyphs should be based
	  * upon.  This is implemented to return a cached
	  * font.
	*/
	public Font getFont()
	{
		/*
		System.out.println("Font ss : "+styleSheet.getStyle("font.name"));
		System.out.println("Real fnt: "+styleSheet.font);
		*/
		return styleSheet.font;
	}

	/**
	  * Determine if the glyphs should be underlined.  If true,
	  * an underline should be drawn through the baseline.
	*/
	public boolean isUnderline()
	{
		return false;
	}

	/**
	  * Determine if the glyphs should have a strikethrough
	  * line.  If true, a line should be drawn through the center
	  * of the glyphs.
	*/
	public boolean isStrikeThrough()
	{
		return false;
	}

	/**
	  * Determine if the glyphs should be rendered as superscript.
	*/
	public boolean isSubscript()
	{
		return false;
	}

	/**
	  * Determine if the glyphs should be rendered as subscript.
	*/
	public boolean isSuperscript()
	{
		return false;
	}

	public float getAlignment(int axis)
	{
		if ( axis==View.X_AXIS )
		{ return styleSheet.align; }
		else
		{ return super.getAlignment(axis);	}
	}

	public short getLeftInset()
	{	return styleSheet.paddingLeft; }

	public short getTopInset()
	{	return styleSheet.paddingTop; }

	public short getBottomInset()
	{	return styleSheet.paddingBottom; }

	public short getRightInset()
	{	return styleSheet.paddingRight; }


	public void setPropertiesFromAttributes()
	{
		styleSheet.update();
	}

	//-------------------------------------------------------------------------
	//
	// Dimensions
	//
	//-------------------------------------------------------------------------

	public float getMaximumSpan(int axis) 
	{
		if ( axis==View.X_AXIS )
		{
			return super.getMaximumSpan(axis)+styleSheet.paddingLeft+
			styleSheet.paddingRight;
		}
		else
		{
			return super.getMaximumSpan(axis)+styleSheet.paddingTop+
			styleSheet.paddingBottom;
		}
	}

	public float getPreferredSpan(int axis)
	{
		if (axis==View.X_AXIS)
		{
			return super.getPreferredSpan(axis)+styleSheet.paddingLeft+
			styleSheet.paddingRight;
		}
		else
		{	
			return super.getPreferredSpan(axis)+styleSheet.paddingTop+
			styleSheet.paddingBottom;
		}
	}

	public float getMinimumSpan(int axis)
	{
		if (axis==View.X_AXIS)
		{
			return super.getMinimumSpan(axis)+styleSheet.paddingLeft+
			styleSheet.paddingRight;
		}
		else
		{	
			return super.getMinimumSpan(axis)+styleSheet.paddingTop+
			styleSheet.paddingBottom;
		}
	}

	public void setSize(float width, float height) 
	{	super.setSize(width,height);	}
	//-------------------------------------------------------------------------
	//
	// Style wrapping
	//
	//-------------------------------------------------------------------------

	public Styleable.StyleSheet getStyleSheet()
	{
		return styleSheet;
	}



	public void setStyleSheet(Styleable.StyleSheet s)
	{
		styleSheet=s;
		styleSheet.update();
	}

	public Styleable.StyleSheet getParentStyleSheet()
	{
		View parent = getParent();
		if ( parent!=null )
		{
			//We try the parent
			if ( parent instanceof Styleable )
			{
				//We try to get the parent stylesheet.
				Styleable.StyleSheet ss = ((Styleable)parent).getStyleSheet();
				//If there is none we try the grandparent one.
				if ( ss==null )
				{	return  ((Styleable)parent).getParentStyleSheet(); }
				else
				{	return ss;	}
			}
			//Or the grandparent (useful with ParagraphView).
			else if ( (parent=parent.getParent())!=null && parent instanceof
			Styleable )
			{
				 Styleable.StyleSheet ss = ((Styleable)parent).getStyleSheet();
				if ( ss==null )
				{	return  ((Styleable)parent).getParentStyleSheet(); }
				else
				{	return ss;	}
			}
			else
			{	return null;	}
		}
		else
		{
			return null;
		}
	}

}

// EOF-Unix/ASCII-------------------------------------@RisingSun//Java//1.0//EN