// ---------------------------------------------------@RisingSun//Java//1.0//EN
// Project			:	SPEdit-Prototypes
// Module			:	GUI-Views
// ClassName		:	BlockView
// Version			:	0.50
// Type				:	
//
// ThreadSafe		:	Nope
// Description		:	A Wrapper for views
//						
// URL				:	<http://SPEdit.sourceforge.net>
// ----------------------------------------------------------------------------
// Authors			:	Sbastien Pierre				<spierre@isb-sib.ch>
// ----------------------------------------------------------------------------
// Creation date	:	19-Oct-2001	
// Last mod.		:	09-Nov-2001	
// History			:
//						09-Nov-2001 Implemented event propagation and border
//						display.(sp)
//						02-Nov-2001 Implemented collapsable.(sp)
//						29-Oct-2001 Improved getParentStyleSheet. (sp)
//						26-Oct-2001 Tooltip enabled.
//						24-Oct-2001 Support selected and focused states.(sp)
//						19-Oct-2001 First implementation. (SP)
//
// Notes			:
//
// Bugs				:
//					  -
//
// To do			:
//					  - 

package net.sourceforge.spedit.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.lang.reflect.*;
import java.util.*;
import java.text.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.ImageIcon;
import net.sourceforge.spedit.core.*;

//-----------------------------------------------------------------------------
//
// WrapperView class
//
//-----------------------------------------------------------------------------

/**
  * <p>The <code>WrapperView</code> class allows to wrap any view and add CSS-like
  * display properties to it. This works very well with views such as
  * <code>ParagraphView</code> or <code>BoxView</code>.</p>
  * <p>The <code>XMLViewFactory</code> of the core package always wrap created
  * views in a <code>WrapperView</code>, so as long as you use the core classes
  * you can assume assume safely that all the views are <code>WrapperView</code>
  * instances (excepted the root view).</p>
  * @version 0.80
  * @author Sbastien Pierre
*/  
public class WrapperView extends View implements Styleable,Stateable
{
	// PUBLIC__________________________________________________________________

	// PROTECTED_______________________________________________________________

	protected View			nested;
	protected AttributesDisplay attributesDisplay;
	protected String		nestedName;
	protected String		title;
	protected Element		element;
	protected StyleSheet	styleSheet;

	protected boolean		focused;
	protected boolean		selected;
	protected boolean		collapsed=false;
	protected Font			collapsedFont = new Font("Helvetica",Font.BOLD,10);
	protected boolean		hasToUpdate=true;
	
	protected Rectangle2D	viewBounds=null;
	protected boolean		refreshBounds=true;
	
	private final Image icn_exp = new ImageIcon("../Support/icon-expanded.gif").getImage();
	private final Image icn_clp = new ImageIcon("../Support/icon-collapsed.gif").getImage();
	
	//-------------------------------------------------------------------------
	//
	// Constructors
	//
	//-------------------------------------------------------------------------

	public WrapperView(View v)
	{
		super(v.getElement());
		title		= null;
		nestedName	= null;
		nested		= v;
		element		= v.getElement();
		attributesDisplay = new AttributesDisplay((XMLElement)element);
		styleSheet	= new StyleSheet("empty", this);
	}

	public WrapperView(Element e, String name)
	{
		super(e);
		nested		= null;
		title		= null;
		nestedName	= name;
		element		= e;
		attributesDisplay = new AttributesDisplay((XMLElement)element);
		styleSheet	= new StyleSheet("empty", this);
	}

	public WrapperView(Element e, String name, String title)
	{
		this(e,name);
		this.title = title;
	}

	//-------------------------------------------------------------------------
	//
	// Painting
	//
	//-------------------------------------------------------------------------

	/**
	*/

	public void paint(Graphics g, Shape s)
	{
		Graphics2D gg=(Graphics2D)g;
		Rectangle2D r = (Rectangle2D)s.getBounds();
		
		if ( getStartOffset()>=getEndOffset()-1 )
		{	g.setColor(new Color(200,200,150));
			gg.fill(r);	}
		
		//We udpate the view bounds
		if ( refreshBounds )
		{	viewBounds=r; refreshBounds=false;	}
		
		//We paint the nested view
		if (!collapsed)
		{	nested.paint(g,s); }

		//And we draw the wrapper attributes
		if (selected)
		{
			g.setColor(new Color(100,200,150,50));
			gg.fill(r);
		}

		if (focused)
		{
			g.setColor(Color.pink);
			gg.draw(r);
		}	

		//We draw the border
		if ( styleSheet.borderSize>0 || isCollapsed() )
		{
			g.setColor(styleSheet.borderColor);
			g.drawRect((int)r.getX()+styleSheet.borderPaddingLeft,
			(int)r.getY()+styleSheet.borderPaddingRight,
			(int)r.getWidth()-(styleSheet.borderPaddingRight+styleSheet.borderPaddingLeft),
			(int)r.getHeight()-(styleSheet.borderPaddingBottom+styleSheet.borderPaddingTop));
		}
		
		//If the view is collapsed
		if ( isCollapsed() )
		{
			if ( styleSheet.hasExpander )
			{
				g.drawImage(icn_clp, (int)viewBounds.getX()+3, (int)viewBounds.getY()+3, null);
			}
			g.setColor(Color.black);
			g.setFont(collapsedFont);
			g.drawString(" + "+getElement().getName()+" ",
				(int)r.getX()+4,
				(int)(r.getY()+12+(r.getHeight()-12)/2));
		}
		//If the view is expanded
		else
		{
			if ( styleSheet.hasExpander )
			{
				g.drawImage(icn_exp, (int)viewBounds.getX()+3, (int)viewBounds.getY()+3, null);
			}
			
			if ( styleSheet.title!=null )
			{
				g.setColor(Color.black);
				g.setFont(styleSheet.titleFont);
				g.drawString(styleSheet.title,
				(int)r.getX()+styleSheet.titlePaddingLeft,
				(int)r.getY()+styleSheet.titlePaddingTop+styleSheet.titleFont.getSize());
			}
		}
		attributesDisplay.paint(getGraphics());
	}

	public void repaint()
	{
		preferenceChanged(null,true,true);
		Container host = getContainer();
		//We set the bounds to null, then call the repaint of the host container
		//which will cause to call the paint method with a shape that corresponds
		//to the actual size of the WrapperView.
		if ( host!=null ) { refreshBounds=true; host.repaint(); }
	}

	public void setFocused(boolean focused)
	{	
		this.focused=focused;
		repaint();
	}
	
	public boolean isFocused()
	{	return this.focused;	}

	public void setSelected(boolean selected)
	{	
		this.selected=selected;
		repaint();
	}

	public boolean isSelected()
	{	return this.selected;	}
	
	public void collapse()
	{	
		collapsed = true;
		repaint();
	}
	
	public void expand()
	{	
		collapsed = false; 
		repaint();
	}
	
	public boolean isCollapsed()
	{	return collapsed; }
	
	/**
	  * Returns the bounds of this view. The rectangle is mutable, so you
	  * should take care of not modifying it. Note: the returned reference
	  * can be null in case you call getBounds between a repaint request and
	  * the actual request or if the WrapperView has not been assigned a
	  * parent.
	  * @return The rectangle that corresponds to the View bounds.
	*/
	public Rectangle2D getBounds()
	{	return viewBounds;	}
	
	/**
	  * Returns the view that is located at the given coordinates. These
	  * coordinates are relative to the current view upper-left corner,
	  * which has (0,0) coordinates.
	  * @return The given WrapperView instance if it exists, otherwise
	  * returns null if there is no view at this location.
	*/
	public WrapperView getViewAt(float x, float y)
	{
		Rectangle2D bounds = (Rectangle2D)getBounds();
		Rectangle2D child_bounds;
		
		View view;
		WrapperView wrapper;
		int view_index;

		for ( int i=0 ; i < getViewCount() ; i++ )
		{
			view = getView(i);
			//We are only interested in wrapper view.
			if ( view!=null && view instanceof WrapperView )
			{
				wrapper = (WrapperView)view;
				child_bounds = wrapper.getBounds();
				if ( child_bounds!=null && child_bounds.contains(x,y) )
				{
					WrapperView child_wrapper = wrapper.getViewAt(x,y);
					if ( child_wrapper!=null )
					{	return child_wrapper;	}
					else
					{	return wrapper;	}
				}
			}
		}
		return null;
	}
	
	//-------------------------------------------------------------------------
	//
	// Event handling
	//
	//-------------------------------------------------------------------------

	/**
	  * <p>Propagates the given event to the nested view if it is a
	  * <code>MouseListener</code> instance. If it is not the case then the
	  * evenet is forwarded to the parent view, which has to be not null
	  * (of course) but also a <code>WrapperView</code> instance.</p>
	  * <p>Note that this particular propagator intercepts the event in case
	  * the event occured on the "expander" icon if it has been set in
	  * the stylesheet.</p>
	  * @param e The mouse event to be propagated.
	*/
	public void propagateMouseClicked(MouseEvent e)
	{
		if ( styleSheet.hasExpander &&
		    (e.getX()-viewBounds.getX())<13 && (e.getY()-viewBounds.getY())<13 )
		{	
			if ( isCollapsed() )
			{	expand();	}
			else
			{	collapse();	}
			return;
		}
		
		if ( nested instanceof MouseListener )
		{
			((MouseListener)nested).mouseClicked(e);
		}
		else if ( getParent()!=null && getParent() instanceof WrapperView )
		{
			((WrapperView)getParent()).propagateMouseClicked(e);
		}
	}
	
	/**
	  * <p>Propagates the given event to the nested view if it is a
	  * <code>MouseListener</code> instance. If it is not the case then the
	  * evenet is forwarded to the parent view, which has to be not null
	  * (of course) but also a <code>WrapperView</code> instance.</p>
	  * @param e The mouse event to be propagated.
	*/
	public void propagateMousePressed(MouseEvent e)
	{
		if ( nested instanceof MouseListener )
		{
			((MouseListener)nested).mousePressed(e);
		}
		else if ( getParent()!=null && getParent() instanceof WrapperView )
		{
			((WrapperView)getParent()).propagateMousePressed(e);
		}
	}
	
	/**
	  * <p>Propagates the given event to the nested view if it is a
	  * <code>MouseListener</code> instance. If it is not the case then the
	  * evenet is forwarded to the parent view, which has to be not null
	  * (of course) but also a <code>WrapperView</code> instance.</p>
	  * @param e The mouse event to be propagated.
	*/
	public void propagateMouseReleased(MouseEvent e)
	{
		if ( nested instanceof MouseListener )
		{
			((MouseListener)nested).mouseReleased(e);
		}
		else if ( getParent()!=null && getParent() instanceof WrapperView )
		{
			((WrapperView)getParent()).propagateMouseReleased(e);
		}
	}
	
	/**
	  * <p>Propagates the given event to the nested view if it is a
	  * <code>MouseListener</code> instance. If it is not the case then the
	  * evenet is forwarded to the parent view, which has to be not null
	  * (of course) but also a <code>WrapperView</code> instance.</p>
	  * @param e The mouse event to be propagated.
	*/
	public void propagateMouseEntered(MouseEvent e)
	{
		if ( nested instanceof MouseListener )
		{
			((MouseListener)nested).mouseEntered(e);
		}
		else if ( getParent()!=null && getParent() instanceof WrapperView )
		{
			((WrapperView)getParent()).propagateMouseEntered(e);
		}
	}
	
	/**
	  * <p>Propagates the given event to the nested view if it is a
	  * <code>MouseListener</code> instance. If it is not the case then the
	  * evenet is forwarded to the parent view, which has to be not null
	  * (of course) but also a <code>WrapperView</code> instance.</p>
	  * @param e The mouse event to be propagated.
	*/
	public void propagateMouseExited(MouseEvent e)
	{
		if ( nested instanceof MouseListener )
		{
			((MouseListener)nested).mouseExited(e);
		}
		else if ( getParent()!=null && getParent() instanceof WrapperView )
		{
			((WrapperView)getParent()).propagateMouseExited(e);
		}
	}
	
	//-------------------------------------------------------------------------
	//
	// Model-View
	//
	//-------------------------------------------------------------------------


	public int viewToModel(float x, float y, Shape a) 
	{	return nested.viewToModel(x,y, a);	}

	public int viewToModel(float x, float y, Shape a, Position.Bias[] biasReturn) 
	{	return nested.viewToModel(x,y,a,biasReturn);	}

	public Shape modelToView(int p0, Position.Bias b0, int p1, Position.Bias b1, Shape a) 
	throws BadLocationException
	{	return nested.modelToView(p0,b0,p1,b1,a);	}

	public Shape modelToView(int pos, Shape a) 
	throws BadLocationException
	{	return nested.modelToView(pos,a);	}

	public Shape modelToView(int pos, Shape a, Position.Bias b) 
	throws BadLocationException
	{	return nested.modelToView(pos,a,b);	}

	//-------------------------------------------------------------------------
	//
	// Dimensions
	//
	//-------------------------------------------------------------------------

	public float getPreferredSpan(int axis)
	{
		//We check wether the wrapper view is collapsed or not
		if (!collapsed)
		{
			if (axis==View.X_AXIS)
			{	return nested.getPreferredSpan(axis);	}
			else
			{	return nested.getPreferredSpan(axis);	}
		}
		//If it is collapsed, then we return another size
		else
		{
			if (axis==View.X_AXIS)
			{	return nested.getPreferredSpan(axis);	}
			else
			{	return Math.max(15,styleSheet.paddingTop+styleSheet.paddingBottom);	}
		}
	}

	public float getMinimumSpan(int axis)
	{
		//We check wether the wrapper view is collapsed or not
		if (!collapsed)
		{
			if (axis==View.X_AXIS)
			{	return nested.getMinimumSpan(axis);	}
			else
			{	return nested.getMinimumSpan(axis);	}
		}
		//If it is collapsed, then we return another size
		else
		{
			if (axis==View.X_AXIS)
			{	return nested.getMinimumSpan(axis);	}
			else
			{	return Math.max(15,styleSheet.paddingTop+styleSheet.paddingBottom);	}
		}
	}
	
	public float getMaximumSpan(int axis) 
	{		
		if ( getStartOffset()>=getEndOffset()-1 )
		{	return 10;	}
		
		//We check wether the wrapper view is collapsed or not
		if (!collapsed)
		{
			if (axis==View.X_AXIS)
			{	return nested.getMaximumSpan(axis);	}
			else
			{	return nested.getMaximumSpan(axis);	}
		}
		//If it is collapsed, then we return another size
		else
		{
			if (axis==View.X_AXIS)
			{	return nested.getMaximumSpan(axis);	}
			else
			{	return Math.max(15,styleSheet.paddingTop+styleSheet.paddingBottom);	}
		}
	}
	

	public void setSize(float width, float height) 
	{	nested.setSize(width,height);	}
	
	//-------------------------------------------------------------------------
	//
	// View management
	//
	//-------------------------------------------------------------------------

	public void append(View v) 
	{	nested.append(v);	}

	public View getParent() 
	{	return nested.getParent();	}

	public View getNestedView()
	{	return nested;	}

	public void replace(int offset, int length, View[] views) 
	{	nested.replace(offset,length,views);	}

	public void setParent(View parent) 
	{	
		if ( nested==null )
		{
			//We create the nested view
			XMLViewFactory fact = (XMLViewFactory)parent.getViewFactory();
			nested = fact.create(element,nestedName);
			if (nested==null)
			{
				System.err.println("WrapperView: No preconfigured view name '"
				+nestedName+"'.");
				nested = new BoxView(element, View.Y_AXIS);
			}
			System.out.println("Setting parent and nested : "+nested);
			nested.setParent(parent);
		
		}
		//This automatically updates the nested view stylesheet
		if ( hasToUpdate )
		{	setStyleSheet(styleSheet); hasToUpdate=false;}
		
		if ( parent!=nested.getParent() )
		{	nested.setParent(parent);}
	}

	public void remove(int i) 
	{	nested.remove(i);	}

	public void removeAll() 
	{	nested.removeAll();	}

	public int getViewIndex(int pos, Position.Bias b) 
	{	return nested.getViewIndex(pos,b);	}

	public void insert(int offs, View v) 
	{	nested.insert(offs,v);	}

	public View getView(int n) 
	{	return nested.getView(n);	}

	public int getViewCount() 
	{	return nested.getViewCount();	}

	public ViewFactory getViewFactory() 
	{	return nested.getViewFactory();	}

	//-------------------------------------------------------------------------
	//
	// View flowing
	//
	//-------------------------------------------------------------------------

	public View breakView(int axis, int offset, float pos, float len) 
	{	return nested.breakView(axis, offset, pos, len);	}

	public View createFragment(int p0, int p1) 
	{	return nested.createFragment(p0, p1);	}

	//-------------------------------------------------------------------------
	//
	// Nested view wrapper
	//
	//-------------------------------------------------------------------------

	public void changedUpdate(DocumentEvent e, Shape a, ViewFactory f) 
	{	nested.changedUpdate(e, a, f);	}

	public float getAlignment(int axis) 
	{	return nested.getAlignment(axis);	}

	public AttributeSet getAttributes() 
	{	return nested.getAttributes();	}

	public int getBreakWeight(int axis, float pos, float len) 
	{	return nested.getBreakWeight(axis,pos,len);	}

	public Shape getChildAllocation(int index, Shape a) 
	{	return nested.getChildAllocation(index, a);	}

	public Container getContainer() 
	{	return nested.getContainer();	}

	public Document getDocument() 
	{	return nested.getDocument();	}

	public Element getElement() 
	{	return nested.getElement();	}

	public int getEndOffset() 
	{	return nested.getEndOffset();	}

	public Graphics getGraphics() 
	{	return nested.getGraphics();	}



	public int getNextVisualPositionFrom(int pos, Position.Bias b, Shape a, int direction,
	Position.Bias[] biasRet) 
	throws BadLocationException
	{	return nested.getNextVisualPositionFrom(pos,b,a,direction,biasRet);	}

	public int getResizeWeight(int axis) 
	{	return nested.getResizeWeight(axis);	}

	public int getStartOffset()
	{	return nested.getStartOffset();	}

	public void insertUpdate(DocumentEvent e, Shape a, ViewFactory f) 
	{	nested.insertUpdate(e,a,f);	}

	public boolean isVisible() 
	{	return nested.isVisible();	}

	public void preferenceChanged(View child, boolean width, boolean height) 
	{	nested.preferenceChanged(child,width,height);	}

	public void removeUpdate(DocumentEvent e, Shape a, ViewFactory f) 
	{	nested.removeUpdate(e,a,f);	}

	String getToolTipText(float x, float y, Shape allocation) 
	{	return nested.getElement().getName(); }

	//-------------------------------------------------------------------------
	//
	// StyleSheet
	//
	//-------------------------------------------------------------------------

	public Styleable.StyleSheet getStyleSheet()
	{
		return styleSheet;
	}

	public void setStyleSheet(Styleable.StyleSheet s)
	{
		styleSheet=s;
		if ( nested instanceof Styleable )
		{
			((Styleable)nested).setStyleSheet(s);
			s.update();
		}
	}

	public Styleable.StyleSheet getParentStyleSheet()
	{
		View parent = getParent();
		if ( parent!=null )
		{
			//We try the parent
			if ( parent instanceof Styleable )
			{
				//We try to get the parent stylesheet.
				Styleable.StyleSheet ss = ((Styleable)parent).getStyleSheet();
				//If there is none we try the grandparent one.
				if ( ss==null )
				{	return  ((Styleable)parent).getParentStyleSheet(); }
				else
				{	return ss;	}
			}
			//Or the grandparent (useful with ParagraphView).
			else if ( (parent=parent.getParent())!=null && parent instanceof
			Styleable )
			{
				 Styleable.StyleSheet ss = ((Styleable)parent).getStyleSheet();
				if ( ss==null )
				{	return  ((Styleable)parent).getParentStyleSheet(); }
				else
				{	return ss;	}
			}
			else
			{	return null;	}
		}
		else
		{
			hasToUpdate = true;
			return null;
		}
	}


}
// EOF-Unix/ASCII-------------------------------------@RisingSun//Java//1.0//EN
