# --------------------------------------------------@RisingSun//Jython//1.0//EN
# Module			:	SPEdit-GUI Prototype
# Version			:	0.01
# Type				:	GUI utility
# ThreadSafe		:	-
# Description		:
# URL				:	<http://SPEdit.sourceforge.net>
# -----------------------------------------------------------------------------
# Author			:	Sebastien Pierre				<spierre@isb-sib.ch>
# -----------------------------------------------------------------------------
# Creation date		:	19-Sep-2001
# Last mod.			:	19-Sep-2001
# History			:	
#						19-Sep-2001 First implementation.
#
# Bugs				:
#						-
#
# To do				:
#						-Test suite

import java, javax, sys
from javax.swing import *

menus = [
	['File',
		(JMenuItem('New...'), "file_new"),
		(JMenuItem('Open...'), "file_open"),
		(JMenuItem('Save...'), "file_save"),
		JSeparator(),
		(JMenuItem('Quit'), "file_quit")
	],
	['Edit',
		(JMenuItem('Cut'), "edit_cut"),
		(JMenuItem('Copy'), "edit_copy"),
		(JMenuItem('Paste'), "edit_paste"),
		JSeparator(),
		(JMenuItem('Preferences...'), "edit_prefs")
	],
	['Tools',
		(JMenuItem('Select'), "tools_select"),
		(JMenuItem('View source...'), "view_source")
	],
	['Windows',
		(JMenuItem('Navigation'), "window_nav"),
		JSeparator()
	],
	['Help',
		(JMenuItem('About'), "help_about"),
		JSeparator(),
		(JMenuItem('Manual'), "help_manual")
	]
]
	
def buildMenus( menus, listener ):
	"""Generates a JMenuBar from the given menu hierarchy."""
	menubar = JMenuBar()
	font = java.awt.Font("Helvetica",java.awt.Font.PLAIN,10)
	menubar.setFont(font)
	for content in menus:
		menu = JMenu(content[0])
		menu.setFont(font)
		for item in content[1:]:
			#Is it a menu item?
			if type(item)==type(()):
				item[0].setActionCommand(item[1])
				item[0].addActionListener(listener)
				menu.add(item[0])
				item[0].setFont(font)
			else:
				menu.add(item)
				item.setFont(font)
		menubar.add(menu)
	return menubar



class MenuListener( java.awt.event.ActionListener ):

	def __init__( self, documentWindow ):
		self.docw = documentWindow
		self.handlers = {
			"file_open": self.openFile,
			"file_save": self.saveFile,
			"file_quit": self.quit,
			"help_manual": self.manual,
			"window_nav": self.showNavigation,
			"view_source": self.viewSource,
		}
	
	def actionPerformed( self, actionEvent ):
		command = actionEvent.getActionCommand()
		if command in self.handlers.keys():
			apply(self.handlers[command],())
	
	def openFile( self ):
		chooser =  JFileChooser()
		returnVal = chooser.showOpenDialog(self.docw)
		if returnVal == JFileChooser.APPROVE_OPTION:
			file = chooser.getSelectedFile()
			doc  = XMLDocument.loadFromFile(file)
			self.docw.setDocument(doc)

	def saveFile( self ):
		chooser =  JFileChooser()
		returnVal = chooser.showSaveDialog(self.docw)
		if returnVal == JFileChooser.APPROVE_OPTION:
			file = chooser.getSelectedFile()
			self.docw.saveDocument(file)

	def manual( self ):
		ed = JEditorPane("file:../Documentation/index.html")
		fr = JFrame("SPEdit - Manual")
		fr.getContentPane().add(ed)
		fr.setBounds(10,20,300,300)
		fr.setVisible(1)

	def quit( self ):
		sys.exit()

	def viewSource( self ):
		f = JFrame("XML Source")
		font  = java.awt.Font("Verdana",java.awt.Font.PLAIN,11)
		f.setBounds(0,0,200,400)
		source = JTextArea(self.docw.getDocument().toString())
		source.setFont(font)
		source.setLineWrap(1)
		f.getContentPane().add(JScrollPane(source))
		f.setVisible(1)

	def showNavigation( self ):
		f = JFrame("Navigation")
		font  = java.awt.Font("Helvetica",java.awt.Font.PLAIN,9)
		f.setBounds(0,0,200,400)
		tree = JTree(self.docw.getDocument())
		tree.setFont(font)
		f.getContentPane().add(JScrollPane(tree))
		f.setVisible(1)

class ElementItem ( JButton, java.awt.event.MouseListener):
	"""The ElementItem represents an element in the hierarchy of elements
	displayed in the status bar. It allows selection of elements that are not
	selectable otherwise, as addition of new elements."""

	def __init__( self, element, viewFactory, docframe):
		JButton.__init__(self, element.getName())
		self.viewFactory = viewFactory
		self.element = element
		self.docframe = docframe
		self.plain  = java.awt.Font("Helvetica",java.awt.Font.PLAIN,9)
		self.bold  = java.awt.Font("Helvetica",java.awt.Font.BOLD,9)
		self.setFont(self.plain)
		self.setHorizontalAlignment(SwingConstants.LEFT)
		self.setMargin(java.awt.Insets(0,0,0,0))
		self.setBorderPainted(0)
		self.addMouseListener(self)

	def mouseClicked(self, e):
		if SwingUtilities.isRightMouseButton(e) or e.isControlDown():
			self.popUpMenu(e)
		else:
			view = self.viewFactory.getElementView(self.element)
			try:
				#view.setSelected(not view.isSelected())
				if view.isCollapsed():
					view.expand()
				else:
					view.collapse()
			except:
				pass	

	def mouseEntered(self, e):
		view = self.viewFactory.getElementView(self.element)
		try:
			self.setFont(self.bold)
			view.setFocused(1)
		except:
			pass

	def mouseExited(self, e):
		view = self.viewFactory.getElementView(self.element)
		try:
			self.setFont(self.plain)
			view.setFocused(0)
		except:
			pass

	def mousePressed(self, e):
		pass

	def mouseReleased(self, e):
		pass

	def popUpMenu(self, e):
		menu = self.docframe.getElementMenu(self.element, self.element.getEndOffset()-1)
		menu.show(self, e.getX(), e.getY())

class StatusBar ( JPanel, javax.swing.event.CaretListener,\
javax.swing.event.ChangeListener ):

	def __init__(self, parent):
		JPanel.__init__(self)
		self.setLayout(java.awt.FlowLayout(java.awt.FlowLayout.LEFT,0,0))
		self._parent = parent
		self._parent.getEditorPane().getCaret().addChangeListener(self)
		self._parent.getEditorPane().addCaretListener(self)
		self.element = "/"
		self.setName("status")
		self.view = None
		self.caretLabel = JLabel()
		bold  = java.awt.Font("Helvetica",java.awt.Font.BOLD,9)
		self.caretLabel.setFont(bold)

	def caretUpdate( self, caretEvent ):
		self.updateCaret(caretEvent.getDot(), caretEvent.getMark())

	def stateChanged( self, stateEvent ):
		pass

	def updateCaret( self, dot, mark ):
		if dot==mark:
			self.caret = "[ %s ]" % (dot)
		else:
			self.caret = "[ %s-%s ]" % ( mark, dot)
		element = self._parent.getDocument().getElementAt(dot) 
		self.caret += " <%s,%s>" % (element.getStartOffset(), element.getEndOffset())
		self.genElement( element )
		self.caretLabel.setText( self.element + "   " + self.caret )
		# We highlight the currently selected view
		old = self.view
		viewFactory=self._parent.getEditorPane().getEditorKit().getViewFactory()
		self.view = viewFactory.getElementView(element.getParent())
		#if old!=self.view:
		#	if old!=None: old.setFocused(0)
		#	self.view.setFocused(1)

	def genElement( self, element ):
		self.removeAll()
		elements = []
		#We create a list of the element parents (reverse)
		while element!=None:
			elements.append(element)
			element = element.getParentElement()
		elements.reverse()
		viewFactory=self._parent.getEditorPane().getEditorKit().getViewFactory()
		for element in elements[:-1]:
			self.add(ElementItem(element, viewFactory, self._parent))
		self.add(self.caretLabel)
		self.doLayout()
		self.repaint()

# EOF-Unix/ASCII------------------------------------@RisingSun//Jython//1.0//EN
